#!/usr/bin/env python
# Copyright (C) 2011-2014 Swift Navigation Inc.
# Contact: Colin Beighley <colin@swift-nav.com>
#
# This source is subject to the license found in the file 'LICENSE' which must
# be be distributed together with this source. All other rights reserved.
#
# THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
# EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.

"""
The :mod:`piksi_tools.acq_results` module contains functions related to
monitoring acquisition.
"""

import serial_link
import argparse
import sys
import time
import struct

from numpy              import mean
from sbp.acquisition    import SBP_MSG_ACQ_RESULT, MsgAcqResult
from sbp.piksi          import SBP_MSG_PRINT
from sbp.client.handler import *

N_RECORD = 0 # Number of results to keep in memory, 0 = no limit.
N_PRINT = 32

SNR_THRESHOLD = 25

class AcqResults():
  """
  AcqResults

  The :class:`AcqResults` collects acquisition results.
  """
  def __init__(self, link):
    self.acqs = []
    self.link = link
    self.link.add_callback(self._receive_acq_result, SBP_MSG_ACQ_RESULT)
    self.max_corr = 0

  def __str__(self):
    tmp = "Last %d acquisitions:\n" % len(self.acqs[-N_PRINT:])
    for a in self.acqs[-N_PRINT:]:
      tmp += "PRN %2d, SNR: %3.2f\n" % (a.prn, a.snr)
    tmp += "Max SNR         : %3.2f\n" % (self.max_snr())
    tmp += "Mean of max SNRs: %3.2f\n" % (self.mean_max_snrs(SNR_THRESHOLD))
    return tmp

  # Return the maximum SNR received.
  def max_snr(self):
    try:
      return max([a.snr for a in self.acqs])
    except ValueError, KeyError:
      return 0

  # Return the mean of the max SNR (above snr_threshold) of each PRN.
  def mean_max_snrs(self, snr_threshold):
    snrs = []
    # Get the max SNR for each PRN.
    for prn in set([a.prn for a in self.acqs]):
      acqs_prn = filter(lambda x: x.prn == prn, self.acqs)
      acqs_prn_max_snr = max([a.snr for a in acqs_prn])
      if acqs_prn_max_snr >= snr_threshold:
        snrs += [max([a.snr for a in acqs_prn])]
    if snrs:
      return mean(snrs)
    else:
      return 0

  def _receive_acq_result(self, sbp_msg):
    while N_RECORD > 0 and len(self.acqs) >= N_RECORD:
      self.acqs.pop(0)
    self.acqs.append(MsgAcqResult(sbp_msg))

def get_args():
  """
  Get and parse arguments.
  """
  import argparse
  parser = argparse.ArgumentParser(description='Acquisition Monitor')
  parser.add_argument("-f", "--ftdi",
                      help="use pylibftdi instead of pyserial.",
                      action="store_true")
  parser.add_argument('-p', '--port',
                      default=[serial_link.SERIAL_PORT], nargs=1,
                      help='specify the serial port to use.')
  parser.add_argument("-b", "--baud",
                      default=[serial_link.SERIAL_BAUD], nargs=1,
                      help="specify the baud rate to use.")
  return parser.parse_args()

def main():
  """
  Get configuration, get driver, and build handler and start it.
  """
  args = get_args()
  port = args.port[0]
  baud = args.baud[0]
  use_ftdi = args.ftdi
  # Driver with context
  with serial_link.get_driver(use_ftdi, port, baud) as driver:
    # Handler with context
    with Handler(driver.read, driver.write) as link:
      link.add_callback(serial_link.printer, SBP_MSG_PRINT)
      acq_results = AcqResults(link)
      link.start()

      try:
        while True:
          print acq_results
          time.sleep(0.1)
      except KeyboardInterrupt:
        pass

if __name__ == "__main__":
  main()



# Copyright (C) 2011-2014 Swift Navigation Inc.
# Contact: Fergus Noble <fergus@swift-nav.com>
#
# This source is subject to the license found in the file 'LICENSE' which must
# be be distributed together with this source. All other rights reserved.
#
# THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
# EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.

from traits.api import Str, Instance, Dict, HasTraits, Array, Float, on_trait_change, Button, Int
from traitsui.api import Item, View, HSplit, VSplit, VGroup

import threading
import time
import struct

from almanac import Almanac
from sbp.piksi import SBP_MSG_ALMANAC, SBP_MSG_SET_TIME

class AlmanacView(HasTraits):
  python_console_cmds = Dict()
  download_almanac = Button(label='Download latest alamanc')
  load_almanac = Button(label='Load alamanc from file')
  send_alm = Button(label='Send almanac to Piksi')
  send_time = Button(label='Send time to Piksi')
  alm = Instance(Almanac)
  alm_txt = Str

  traits_view = View(
  	VGroup(
	    Item('alm_txt', label='PRNs visible'),
	    Item('download_almanac'),
	    Item('load_almanac'),
	    Item('send_alm'),
	    Item('send_time')
  	)
  )

  def _send_time_fired(self):
    gps_secs = time.time() - (315964800 - 16)
    gps_week = int(gps_secs / (7*24*3600))
    gps_tow = gps_secs % (7*24*3600)
    print gps_week, gps_tow
    buff = struct.pack("<dH", gps_tow, gps_week)
    self.link.send(SBP_MSG_SET_TIME, buff)

  def _send_alm_fired(self):
    self.update_alamanc_view()

    for sat in self.alm.sats:
      self.link.send(SBP_MSG_ALMANAC, sat.packed())
      time.sleep(0.1)

  def update_alamanc_view(self):
    #prns, dopps = zip(*self.alm.get_dopps())
    self.alm_txt = str(self.alm.get_dopps())

  def _download_almanac_fired(self):
    self.alm.download_almanac()
    self.update_alamanc_view()

  def _load_almanac_fired(self):
    self.alm.load_almanac_file('current.alm')
    self.update_alamanc_view()

  def __init__(self, link):
    super(AlmanacView, self).__init__()

    self.link = link

    self.alm = Almanac()

    self.python_console_cmds = {
      'alm': self.alm
    }




#!/usr/bin/env python
# Copyright (C) 2011-2014 Swift Navigation Inc.
# Contact: Niels Joubert <njoubert@gmail.com>
#
# This source is subject to the license found in the file 'LICENSE' which must
# be be distributed together with this source. All other rights reserved.
#
# THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
# EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.

#This script resets the FDTI settings to its default values

import serial_link
import time
import struct
import argparse
import sys

from sbp.piksi import SBP_MSG_SETTINGS, SBP_MSG_SETTINGS_SAVE

def send_setting(link, section, name, value):
  link.send(SBP_MSG_SETTINGS, '%s\0%s\0%s\0' % (section, name, value))


def get_args():
  """
  Get and parse arguments.
  """
  import argparse
  parser = argparse.ArgumentParser(description='Piksi Recover FTDI')
  parser.add_argument('-p', '--port',
                      default=[serial_link.SERIAL_PORT], nargs=1,
                      help='specify the serial port to use.')
  parser.add_argument("-b", "--baud",
                      default=[serial_link.SERIAL_BAUD], nargs=1,
                      help="specify the baud rate to use.")
  parser.add_argument("-f", "--ftdi",
                      help="use pylibftdi instead of pyserial.",
                      action="store_true")
  return parser.parse_args()

def main():
  """
  Get configuration, get driver, and build handler and start it.
  """
  port = args.port[0]
  baud = args.baud[0]
  # Driver with context
  with serial_link.get_driver(args.ftdi, port, baud) as driver:
    # Handler with context
    with Handler(driver.read, driver.write) as link:
      print "Resetting mask to 0xff"
      send_setting(link, "uart_ftdi", "sbp_message_mask", "65535")
      time.sleep(0.5)

      print "Resetting baudrate to 1Mbps"
      send_setting(link, "uart_ftdi", "baudrate", "1000000")
      time.sleep(0.5)

      print "Resetting mode to SBP"
      send_setting(link, "uart_ftdi", "mode", "SBP")
      time.sleep(0.5)

      print "Attempting to save settings"
      link.send(SBP_MSG_SETTINGS_SAVE, "")
      time.sleep(0.5)
      link.send(SBP_MSG_SETTINGS_SAVE, "")
      time.sleep(0.5)
      link.send(SBP_MSG_SETTINGS_SAVE, "")
      time.sleep(0.5)
      link.send(SBP_MSG_SETTINGS_SAVE, "")

      print "Sent Settings Reset message to return FTDI to defaults"

if __name__ == "__main__":
  main()



#!/usr/bin/env python
# Copyright (C) 2011-2014 Swift Navigation Inc.
# Contact: Fergus Noble <fergus@swift-nav.com>
#
# This source is subject to the license found in the file 'LICENSE' which must
# be be distributed together with this source. All other rights reserved.
#
# THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
# EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.

import math as m
import numpy as n
import time
import urllib2

NAV_GM = 3.986005e14
NAV_OMEGAE_DOT = 7.2921151467e-005
GPS_L1_HZ = 1.57542e9
NAV_C = 299792458.0

RANDOM_SUNDAY = 1283644784
WPR = (-2712219.0, -4316338.0, 3820996.0)

def time_of_week():
  return (time.time() - RANDOM_SUNDAY) % (7*24*60*60)

class Sat:
  def __init__(self, yuma_block):
    fields = map(lambda x: x[25:], yuma_block)
    self.prn     = int(fields[0])
    self.healthy = (int(fields[1]) == 0)
    self.ecc     = float(fields[2])
    self.toa     = float(fields[3])
    self.inc     = float(fields[4])
    self.rora    = float(fields[5])
    self.a       = float(fields[6])**2
    self.raaw    = float(fields[7])
    self.argp    = float(fields[8])
    self.ma      = float(fields[9])
    self.af0     = float(fields[10])
    self.af1     = float(fields[11])
    self.week    = int(fields[12])

  def calc_vis_dopp(self, time_of_week, receiver_pos, elevation_mask=10.0):
    tdiff = time_of_week - self.toa

    if tdiff > 302400.0:
      tdiff -= 604800.0
    elif tdiff < -302400.0:
      tdiff += 604800.0

    ma_dot = m.sqrt(NAV_GM / self.a**3)
    ma = self.ma + ma_dot*tdiff

    # Iteratively solve for the Eccentric Anomaly (from Keith Alter and David Johnston)
    ea = ma # Starting value for E
    ea_old = ea + 1

    while (abs (ea - ea_old) > 1.0E-14):
      ea_old = ea
      tempd1 = 1.0 - self.ecc * m.cos(ea_old)
      ea = ea + (ma - ea_old + self.ecc * m.sin(ea_old)) / tempd1
    ea_dot = ma_dot / tempd1

    # Begin calc for True Analomay and Argument of Latitude
    tempd2 = m.sqrt(1.0 - self.ecc * self.ecc)
    # [rad] Argument of Latitude = True Anomaly + Argument of Perigee
    al = m.atan2(tempd2 * m.sin(ea), m.cos(ea) - self.ecc) + self.argp
    al_dot = tempd2 * ea_dot / tempd1

    # Calculate corrected radius based on argument of latitude
    r = self.a * tempd1
    r_dot = self.a * self.ecc * m.sin(ea) * ea_dot

    # Calculate inclination based on argument of latitude
    inc = self.inc

    # Calculate position and velocity in orbital plane
    x = r * m.cos(al)
    y = r * m.sin(al)
    x_dot = r_dot * m.cos(al) - y * al_dot
    y_dot = r_dot * m.sin(al) + x * al_dot

    # Corrected longitude of ascending node
    om_dot = self.rora - NAV_OMEGAE_DOT
    om = self.raaw + tdiff * om_dot - NAV_OMEGAE_DOT * self.toa

    sat_pos = n.array([0.0, 0.0, 0.0])
    sat_vel = n.array([0.0, 0.0, 0.0])

    # Compute the satellite's position in Earth-Centered Earth-Fixed coordiates
    sat_pos[0] = x * m.cos(om) - y * m.cos(inc) * m.sin(om)
    sat_pos[1] = x * m.sin(om) + y * m.cos(inc) * m.cos(om)
    sat_pos[2] = y * m.sin(inc)

    tempd3 = y_dot * m.cos(inc)

    # Compute the satellite's velocity in Earth-Centered Eart-Fixed coordiates
    sat_vel[0] = -om_dot * sat_pos[1] + x_dot * m.cos(om) - tempd3 * m.sin(om)
    sat_vel[1] = om_dot * sat_pos[0] + x_dot * m.sin(om) + tempd3 * m.cos(om)
    sat_vel[2] = y_dot * m.sin(inc)

    # Compute the angle that a satellite makes with the horizon for
    # the passed receiver location
    vec_rec_sat = sat_pos - receiver_pos
    vec_rec_sat /= m.sqrt(n.vdot(vec_rec_sat, vec_rec_sat))

    # normalize the vectors to find the angle in between them
    angle_to_normal = m.acos(n.vdot(vec_rec_sat, receiver_pos)/m.sqrt(n.vdot(receiver_pos, receiver_pos)))
    angle_to_horizon = m.pi/2 - angle_to_normal
    # velocity of the satellite in the axis of the vector from receiver to satellite
    radial_velocity = n.vdot(vec_rec_sat, sat_vel)
    doppler_shift = GPS_L1_HZ * -radial_velocity / NAV_C

    if (angle_to_horizon > elevation_mask*(m.pi/180)) and (self.healthy):
      return (doppler_shift, angle_to_horizon)
    else:
      return (None, None)

  def packed(self):
    import struct
    return struct.pack("<ddddddddddHBBB",
      self.ecc, self.toa, self.inc, self.rora, self.a, self.raaw, self.argp,
      self.ma, self.af0, self.af1, self.week, self.prn, self.healthy, 1
    )

  def __str__(self):
    dopp, _ = self.calc_vis_dopp(time_of_week(), WPR)
    if dopp != None:
      return "PRN%02d\t@ %+7.1f Hz\n" % (self.prn, dopp)
    else:
      return "PRN%02d Not Visible\n" % self.prn

class Almanac:
  sats = None

  def almanac_valid(self):
    return (self.sats != None)

  def download_almanac(self):
    u = urllib2.urlopen('http://www.navcen.uscg.gov/?pageName=currentAlmanac&format=yuma')
    if u:
      self.process_yuma(u.readlines())

  def load_almanac_file(self, filename):
    fp = open(filename, 'r')
    if fp:
      self.process_yuma(fp.readlines())
    fp.close()

  def process_yuma(self, yuma):
    if yuma:
      blocks = []
      for (n, line) in enumerate(yuma):
        if line[:3] == "ID:":
          blocks += [yuma[n:n+13]]
      self.sats = map(lambda bl: Sat(bl), blocks)
    else:
      self.sats = None

  def get_dopps(self, tow=None, location=WPR):
    if not tow:
      tow = time_of_week()

    if self.sats:
      dopps = map(lambda s: (s.prn,)+s.calc_vis_dopp(tow, location, elevation_mask=0.0), self.sats)
      dopps = filter(lambda (prn, dopp, el): (dopp != None), dopps)
      return dopps
    else:
      return None

if __name__ == "__main__":
  alm = Almanac()
  print "Downloading current almanac"
  alm.download_almanac()
  print "Dopplers:"
  print alm.get_dopps()



#!/usr/bin/env python
# Copyright (C) 2011-2014 Swift Navigation Inc.
# Contact: Fergus Noble <fergus@swift-nav.com>
#
# This source is subject to the license found in the file 'LICENSE' which must
# be be distributed together with this source. All other rights reserved.
#
# THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
# EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.

"""Contains the class OutputStream, a HasTraits file-like text buffer."""

from traits.api import HasTraits, Str, Bool, Trait, Int
from traitsui.api import View, UItem, TextEditor, Handler
from traits.etsconfig.api import ETSConfig
from pyface.api import GUI


DEFAULT_MAX_LEN = 8000


class _OutputStreamViewHandler(Handler):

    def object_text_changed(self, uiinfo):
        ui = uiinfo.ui
        if ui is None:
            return

        for ed in  ui._editors:
            if ed.name == 'text':
                break
        else:
            # Did not find an editor with the name 'text'.
            return

        if ETSConfig.toolkit == 'wx':
            # With wx, the control is a TextCtrl instance.
            ed.control.SetInsertionPointEnd()
        elif ETSConfig.toolkit == 'qt4':
            # With qt4, the control is a QtGui.QTextEdit instance.
            from pyface.qt.QtGui import QTextCursor
            ed.control.moveCursor(QTextCursor.End)


class OutputStream(HasTraits):
    """This class has methods to emulate an file-like output string buffer.

    It has a default View that shows a multiline TextEditor.  The view will
    automatically move to the end of the displayed text when data is written
    using the write() method.

    The `max_len` attribute specifies the maximum number of bytes saved by
    the object.  `max_len` may be set to None.

    The `paused` attribute is a bool; when True, text written to the
    OutputStream is saved in a separate buffer, and the display (if there is
    one) does not update.  When `paused` returns is set to False, the data is
    copied from the paused buffer to the main text string.
    """

    # The text that has been written with the 'write' method.
    text = Str

    # The maximum allowed length of self.text (and self._paused_buffer).
    max_len = Trait(DEFAULT_MAX_LEN, None, Int)

    # When True, the 'write' method appends its value to self._paused_buffer
    # instead of self.text.  When the value changes from True to False,
    # self._paused_buffer is copied back to self.text.
    paused = Bool(False)

    # String that holds text written while self.paused is True.
    _paused_buffer = Str

    def write(self, s):
        if self.paused:
            self._paused_buffer = self._truncated_concat(self._paused_buffer, s)
        else:
            self.text = self._truncated_concat(self.text, s)

    def flush(self):
        GUI.process_events()

    def close(self):
        pass

    def reset(self):
        self._paused_buffer = ''
        self.paused = False
        self.text = ''

    def _truncated_concat(self, text, s):
        if len(s) >= self.max_len:
            # s could be huge. Handle this case separately, to avoid the temporary
            # created by 'text + s'.
            result = s[-self.max_len:]
        else:
            result = (text + s)[-self.max_len:]
        return result

    def _paused_changed(self):
        if self.paused:
            # Copy the current text to _paused_buffer.  While the OutputStream
            # is paused, the write() method will append its argument to _paused_buffer.
            self._paused_buffer = self.text
        else:
            # No longer paused, so copy the _paused_buffer to the displayed text, and
            # reset _paused_buffer.
            self.text = self._paused_buffer
            self._paused_buffer = ''

    def traits_view(self):
        view = \
            View(
                UItem('text', editor=TextEditor(multi_line=True), style='custom'),
                handler=_OutputStreamViewHandler(),
            )
        return view




#!/usr/bin/env python
# Copyright (C) 2011-2014 Swift Navigation Inc.
# Contact: Fergus Noble <fergus@swift-nav.com>
#          Colin Beighley <colin@swift-nav.com>
#
# This source is subject to the license found in the file 'LICENSE' which must
# be be distributed together with this source. All other rights reserved.
#
# THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
# EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.

import struct
import time
import sys
import new
from intelhex import IntelHex
from threading import Lock

from itertools import groupby
from sbp.piksi import *

# 0.5 * ADDRS_PER_OP * MAX_QUEUED_OPS (plus SBP overhead) is how much each of
# the STM32 TX/RX buffers will be filled by the program/read callback messages.
ADDRS_PER_OP = 128
MAX_QUEUED_OPS = 1

M25_SR_SRWD = 1 << 7
M25_SR_BP2  = 1 << 4
M25_SR_BP1  = 1 << 3
M25_SR_BP0  = 1 << 2
M25_SR_WEL  = 1 << 1
M25_SR_WIP  = 1 << 0

STM_RESTRICTED_SECTORS = [0]
STM_N_SECTORS = 12

M25_RESTRICTED_SECTORS = [15]
M25_N_SECTORS = 16

def stm_addr_sector_map(addr):
  """
  Map an STM32F4 flash address to a sector.

  Parameters
  ----------
  addr : int
      STM flash address.

  Returns
  -------
  out : int
      STM flash sector.
  """
  if   addr >= 0x08000000 and addr < 0x08004000:
    return 0
  elif addr >= 0x08004000 and addr < 0x08008000:
    return 1
  elif addr >= 0x08008000 and addr < 0x0800C000:
    return 2
  elif addr >= 0x0800C000 and addr < 0x08010000:
    return 3
  elif addr >= 0x08010000 and addr < 0x08020000:
    return 4
  elif addr >= 0x08020000 and addr < 0x08040000:
    return 5
  elif addr >= 0x08040000 and addr < 0x08060000:
    return 6
  elif addr >= 0x08060000 and addr < 0x08080000:
    return 7
  elif addr >= 0x08080000 and addr < 0x080A0000:
    return 8
  elif addr >= 0x080A0000 and addr < 0x080C0000:
    return 9
  elif addr >= 0x080C0000 and addr < 0x080E0000:
    return 10
  elif addr >= 0x080E0000 and addr < 0x08100000:
    return 11
  else:
    raise IndexError("Attempted to access flash memory at (%s) outside of range (wrong firmware file?)." % hex(addr))
    return None

def m25_addr_sector_map(addr):
  """
  Map an M25 flash address to a sector.

  Parameters
  ----------
  addr : int
      M25 flash address.

  Returns
  -------
  out : int
      M25 flash sector.
  """
  if addr < 0 or addr > 0xFFFFF:
    raise IndexError("Attempted to access flash memory at (%s) outside of range (wrong firmware file?)." % hex(addr))
  return addr >> 16

def ihx_ranges(ihx):
  """
  Find occupied address ranges in intelhex.IntelHex object.

  Parameters
  ----------
  ihx : intelhex.Intelhex
      intelhex.IntelHex object to find occupied address ranges of.

  Returns
  -------
  out : list[(int, int), (int, int), ...]
      List of min/max tuples of occupied address ranges.
  """
  def first_last(x):
    first = x.next()
    last = first
    for last in x:
      pass
    return (first[1], last[1])
  return [first_last(v) for k, v in
          groupby(enumerate(ihx.addresses()), lambda (i, x) : i - x)]

def sectors_used(addrs, addr_sector_map):
  """
  Given a list of min/max address pairs, find the sectors they occupy.

  Parameters
  ----------
  addrs : list[(int, int), (int, int), ...]
      List of min/max tuples of occupied address ranges to be mapped to sectors.
  addr_sector_map : function
      Function that maps an address to a sector.

  Returns
  -------
  out : list[int]
      List of sectors in which the passed addresses reside in the passed
      address sector map.
  """
  sectors = set()
  for s, e in addrs:
    sectors |= set(range(addr_sector_map(s), addr_sector_map(e)+1))
  return sorted(list(sectors))

def ihx_n_ops(ihx, addr_sector_map, erase=True):
  """
  Find the number of sent SBP messages (erase, program, read) Flash.write_ihx
  will require to write a particular intelhex.IntelHex.

  Parameters
  ----------
  ihx : intelhex.Intelhex
      intelhex.IntelHex to be written.
  addr_sector_map : function
      Function that maps an address to a sector.
  erase : bool
      Include number of erase operations required in total.

  Returns
  -------
  out : int
      Number of sent SBP messages (erase, program, read) Flash.write_ihx will
      require to write ihx.
  """
  ihx_addrs = ihx_ranges(ihx)
  erase_ops = len(sectors_used(ihx_addrs, addr_sector_map))
  program_ops_addr_args = [range(s,e,ADDRS_PER_OP) for s,e in ihx_addrs]
  program_ops = sum([len(l) for l in program_ops_addr_args])
  read_ops = program_ops # One read callback for every program callback.
  if erase:
    return erase_ops + program_ops + read_ops
  return program_ops + read_ops

# Defining separate functions to lock/unlock STM sectors and to read/write M25
# status register, as there isn't a great way to define lock/unlock sector
# callbacks that will be general to both flashes (see M25Pxx datasheet).
# Functions conditionally bound to Flash instance in Flash.__init__(),
# depending on flash_type.

def _stm_lock_sector(self, sector):
  """
  Lock a sector of the STM flash.

  Parameters
  ----------
  sector : int
      Sector of STM flash to lock (> 0, <= 11).
  """
  if not 0 <= sector <+ 11:
    raise ValueError("Must have 0 <= sector <= 11, received %d" % sector)
  msg_buf = struct.pack("B", sector)
  self.inc_n_queued_ops()
  self.link.send(SBP_MSG_STM_FLASH_LOCK_SECTOR, msg_buf)
  while self.get_n_queued_ops() > 0:
    time.sleep(0.001)

def _stm_unlock_sector(self, sector):
  """
  Unlock a sector of the STM flash.

  Parameters
  ----------
  sector : int
      Sector of STM flash to unlock (> 0, <= 11).
  """
  if not 0 <= sector <= 11:
    raise ValueError("Must have 0 <= sector <= 11, received %d" % sector)
  msg_buf = struct.pack("B", sector)
  self.inc_n_queued_ops()
  self.link.send(SBP_MSG_STM_FLASH_UNLOCK_SECTOR, msg_buf)
  while self.get_n_queued_ops() > 0:
    time.sleep(0.001)

def _m25_write_status(self, sr):
  """
  Write to the M25 status register.

  Parameters
  ----------
  sr : int
      Value to write to the M25 status register (> 0, <= 255).
  """
  if not 0 <= sr <= 255:
    raise ValueError("Must have 0 <= sr <= 255, received %d" % sr)
  msg_buf = struct.pack("B", sr)
  self.inc_n_queued_ops()
  self.link.send(SBP_MSG_M25_FLASH_WRITE_STATUS, msg_buf)
  while self.get_n_queued_ops() > 0:
    time.sleep(0.001)

class Flash():

  def __init__(self, link, flash_type):
    """
    Object representing either of the two flashes (STM/M25) on the Piksi,
    including methods to erase, program, and read.

    Parameters
    ----------
    link : sbp.client.handler.Handler
      Handler to send messages to Piksi over and register callbacks with.
    flash_type : string
      Which Piksi flash to interact with ("M25" or "STM").

    Returns
    -------
    out : Flash instance
    """
    self._n_queued_ops = 0
    self.nqo_lock = Lock()
    self.stopped = False
    self.status = ''
    self.link = link
    self.flash_type = flash_type
    # IntelHex object to store read flash data in that was read from device.
    self._read_callback_ihx = IntelHex()
    self.link.add_callback(self._done_callback, SBP_MSG_FLASH_DONE)
    self.link.add_callback(self._read_callback, SBP_MSG_FLASH_READ)
    self.ihx_elapsed_ops = 0 # N operations finished in self.write_ihx
    if self.flash_type == "STM":
      self.flash_type_byte = 0
      self.addr_sector_map = stm_addr_sector_map
      # Add STM-specific functions.
      self.__dict__['lock_sector'] = \
          new.instancemethod(_stm_lock_sector, self, Flash)
      self.__dict__['unlock_sector'] = \
          new.instancemethod(_stm_unlock_sector, self, Flash)
      self.n_sectors = STM_N_SECTORS
      self.restricted_sectors = STM_RESTRICTED_SECTORS
    elif self.flash_type == "M25":
      self.flash_type_byte = 1
      self.addr_sector_map = m25_addr_sector_map
      # Add M25-specific functions.
      self.__dict__['write_status'] = \
          new.instancemethod(_m25_write_status, self, Flash)
      self.n_sectors = M25_N_SECTORS
      self.restricted_sectors = M25_RESTRICTED_SECTORS
    else:
      raise ValueError("flash_type must be \"STM\" or \"M25\", got \"%s\"" \
                       % flash_type)

  def __enter__(self):
    return self

  def __exit__(self, type, value, traceback):
    """ Calls self.stop() before instance is deleted. """
    if not self.stopped:
      self.stop()

  def ihx_n_ops(self, ihx, erase=True):
    """
    Find the number of sent SBP messages (erase, program, read) self.write_ihx
    will require to write a particular intelhex.IntelHex for this instance's
    flash type.

    Parameters
    ----------
    ihx : intelhex.Intelhex
      intelhex.IntelHex to be written.
    erase : bool
      Include number of erase operations required in total.

    Returns
    -------
    out : int
      Number of sent SBP messages (erase, program, read) self.write_ihx will
      require to write ihx.
    """
    return ihx_n_ops(ihx, self.addr_sector_map, erase)

  def inc_n_queued_ops(self):
    """
    Increment the count of queued flash operation SBP messages in the STM's
    flash in a thread safe way.
    """
    self.nqo_lock.acquire()
    self._n_queued_ops += 1
    self.nqo_lock.release()

  def dec_n_queued_ops(self):
    """
    Decrement the count of queued flash operation SBP messages in the STM's
    flash in a thread safe way.
    """
    self.nqo_lock.acquire()
    self._n_queued_ops -= 1
    self.nqo_lock.release()

  def get_n_queued_ops(self):
    """
    Get the current count of queued flash operation SBP messages.

    Returns
    -------
    out : int
      Get the current count of queued flash operation SBP messages.
    """
    return self._n_queued_ops

  def stop(self):
    """ Remove instance callbacks from sbp.client.handler.Handler. """
    self.stopped = True
    self.link.remove_callback(self._done_callback, SBP_MSG_FLASH_DONE)
    self.link.remove_callback(self._read_callback, SBP_MSG_FLASH_READ)

  def __str__(self):
    """ Return flashing status. """
    return self.status

  def erase_sector(self, sector, warn=True):
    """
    Erase a sector of the flash.

    Parameters
    ----------
    sector : int
      Sector to be erased.
    warn : bool
      Warn if sector is restricted and should most likely not be erased.
    """
    if warn and sector in self.restricted_sectors:
      text = 'Attempting to erase %s flash restricted sector %d' % \
             (self.flash_type, sector)
      raise Warning(text)
    msg_buf = struct.pack("BB", self.flash_type_byte, sector)
    self.inc_n_queued_ops()
    self.link.send(SBP_MSG_FLASH_ERASE, msg_buf)
    while self.get_n_queued_ops() > 0:
      time.sleep(0.001)

  def program(self, address, data):
    """
    Program a set of addresses of the flash.

    Parameters
    ----------
    address : int
      Starting address of set of addresses to program with data.
    data : string
      String of bytes to program to flash starting at address.
    """
    msg_buf = struct.pack("B", self.flash_type_byte)
    msg_buf += struct.pack("<I", address)
    msg_buf += struct.pack("B", len(data))
    self.inc_n_queued_ops()
    self.link.send(SBP_MSG_FLASH_PROGRAM, msg_buf + data)

  def read(self, address, length):
    """
    Read a set of addresses of the flash.

    Parameters
    ----------
    address : int
      Starting address of length addresses to read.
    length : int
      Number of addresses to read.
    """
    msg_buf = struct.pack("B", self.flash_type_byte)
    msg_buf += struct.pack("<I", address)
    msg_buf += struct.pack("B", length)
    self.inc_n_queued_ops()
    self.link.send(SBP_MSG_FLASH_READ, msg_buf)

  def _done_callback(self, sbp_msg):
    """
    Handles flash done message sent from device.

    Parameters
    ----------
    data : string
      Binary data sent from device.
    """
    ret = ord(sbp_msg.payload)

    if (ret != 0):
      print "Flash operation returned error (%d)" % ret

    self.dec_n_queued_ops()
    assert self.get_n_queued_ops() >= 0, \
      "Number of queued flash operations is negative"

  def _read_callback(self, sbp_msg):
    """
    Handles flash read message sent from device.

    Parameters
    ----------
    data : string
      Binary data sent from device.
    """
    # 4 bytes addr, 1 byte length, length bytes data
    address = struct.unpack('<I', sbp_msg.payload[0:4])[0]
    length = struct.unpack('B', sbp_msg.payload[4])[0]

    self._read_callback_ihx.puts(address, sbp_msg.payload[5:])

    self.dec_n_queued_ops()
    assert self.get_n_queued_ops() >= 0, \
      "Number of queued flash operations is negative"

  def write_ihx(self, ihx, stream=None, mod_print=0, elapsed_ops_cb=None, erase=True):
    """
    Perform all operations to write an intelhex.IntelHex to the flash
    and verify.

    Parameters
    ----------
    ihx : intelhex.IntelHex
      intelhex.IntelHex to write to the flash.
    stream : stream
      Object implementing write and flush methods to write status updates to.
    mod_print : int
      How many loops to run through before writing to stream.
    elapsed_ops_cb : function
      Callback to execute every mod_print loops.
    erase : bool
      Erase sectors before writing.
    """
    self.ihx_elapsed_ops = 0
    self.print_count = 0

    start_time = time.time()

    ihx_addrs = ihx_ranges(ihx)

    # Erase sectors
    if erase:
      for sector in sectors_used(ihx_addrs, self.addr_sector_map):
        self.status = self.flash_type + " Flash: Erasing sector %d" % sector
        if stream:
          stream.write('\r' + self.status)
          stream.flush()
        self.erase_sector(sector)
        self.ihx_elapsed_ops += 1
        if elapsed_ops_cb != None:
          elapsed_ops_cb(self.ihx_elapsed_ops)
      if stream:
        stream.write('\n')

    # Write data to flash and read back to later validate. STM's lowest address
    # is used by bootloader to check that the application is valid, so program
    # from high to low to ensure this address is programmed last.
    for start, end in reversed(ihx_addrs):
      for addr in reversed(range(start, end, ADDRS_PER_OP)):
        self.status = self.flash_type + " Flash: Programming address" + \
                                        " 0x%08X" % addr
        if mod_print == 0 or mod_print != 0 and self.print_count % mod_print == 0:
          if stream:
            stream.write('\r' + self.status)
            stream.flush()
          self.print_count = 1
          if elapsed_ops_cb != None:
            elapsed_ops_cb(self.ihx_elapsed_ops)
        else:
          self.print_count += 1

        binary = ihx.tobinstr(start=addr, size=ADDRS_PER_OP)

        # Program ADDRS_PER_OP addresses
        while self.get_n_queued_ops() >= MAX_QUEUED_OPS:
          time.sleep(0.001)
        self.program(addr, binary)
        self.ihx_elapsed_ops += 1

        # Read ADDRS_PER_OP addresses
        while self.get_n_queued_ops() >= MAX_QUEUED_OPS:
          time.sleep(0.001)
        self.read(addr, ADDRS_PER_OP)
        self.ihx_elapsed_ops += 1

    # Verify that data written to flash matches data read from flash.
    while self.get_n_queued_ops() > 0:
      time.sleep(0.001)
    for start, end in reversed(ihx_addrs):
      if self._read_callback_ihx.gets(start, end-start+1) != \
          ihx.gets(start, end-start+1):
        for i in range(start, end):
          r = self._read_callback_ihx.gets(i,1)
          p = ihx.gets(i,1)
          if r != p:
            raise Exception('Data read from flash != Data programmed to flash'
              ('Addr: %x, Programmed: %x, Read: %x' % (i,r,p)).upper())

    self.status = self.flash_type + " Flash: Successfully programmed and " + \
                                    "verified, total time = %d seconds" % \
                                    int(time.time()-start_time)
    if stream:
      stream.write('\n\r' + self.status + '\n')




#!/usr/bin/env python
# Copyright (C) 2011-2014 Swift Navigation Inc.
# Contact: Fergus Noble <fergus@swift-nav.com>
#
# This source is subject to the license found in the file 'LICENSE' which must
# be be distributed together with this source. All other rights reserved.
#
# THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
# EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.

from traits.api import Instance, Dict, HasTraits, Array, Float, on_trait_change, List, Int, Button, Bool
from traitsui.api import Item, View, HGroup, VGroup, ArrayEditor, HSplit, TabularEditor, UItem, Tabbed
from traitsui.tabular_adapter import TabularAdapter
from chaco.api import ArrayPlotData, Plot
from chaco.tools.api import ZoomTool, PanTool
from enable.api import ComponentEditor
from enable.savage.trait_defs.ui.svg_button import SVGButton
from pyface.api import GUI

import struct
import math
import os
import numpy as np
import datetime
import time

from sbp.navigation import *

class SimpleAdapter(TabularAdapter):
  columns = [('Item', 0), ('Value',  1)]
  width = 80

class SolutionView(HasTraits):
  python_console_cmds = Dict()
  # we need to doubleup on Lists to store the psuedo absolutes separately
  # without rewriting everything
  lats = List()
  lngs = List()
  alts = List()

  lats_psuedo_abs = List()
  lngs_psuedo_abs = List()
  alts_psuedo_abs = List()

  table_spp = List()
  table_psuedo_abs = List()
  dops_table = List()
  pos_table_spp = List()
  vel_table = List()

  plot = Instance(Plot)
  plot_data = Instance(ArrayPlotData)
  # Store plots we care about for legend

  running = Bool(True)
  position_centered = Bool(False)

  clear_button = SVGButton(
    label='', tooltip='Clear',
    filename=os.path.join(os.path.dirname(__file__), 'images', 'iconic', 'x.svg'),
    width=16, height=16
  )
  zoomall_button = SVGButton(
    label='', tooltip='Zoom All',
    filename=os.path.join(os.path.dirname(__file__), 'images', 'iconic', 'fullscreen.svg'),
    width=16, height=16
  )
  center_button = SVGButton(
    label='', tooltip='Center on Solution', toggle=True,
    filename=os.path.join(os.path.dirname(__file__), 'images', 'iconic', 'target.svg'),
    width=16, height=16
  )
  paused_button = SVGButton(
    label='', tooltip='Pause', toggle_tooltip='Run', toggle=True,
    filename=os.path.join(os.path.dirname(__file__), 'images', 'iconic', 'pause.svg'),
    toggle_filename=os.path.join(os.path.dirname(__file__), 'images', 'iconic', 'play.svg'),
    width=16, height=16
  )

  traits_view = View(
    HSplit(
      Tabbed(
        VGroup(
          Item('', label='Single Point Position (SPP)', emphasized=True),
          Item('table_spp', style='readonly',
                editor=TabularEditor(adapter=SimpleAdapter()),
                show_label=False, width=0.3),
          label='Single Point Position'),
        VGroup(
          Item('', label='RTK Position', emphasized=True),
          Item('table_psuedo_abs',style='readonly',
               editor=TabularEditor(adapter=SimpleAdapter()),
               show_label=False, width=0.3),
          label='RTK Position')
      ),
      VGroup(
        HGroup(
          Item('paused_button', show_label=False),
          Item('clear_button', show_label=False),
          Item('zoomall_button', show_label=False),
          Item('center_button', show_label=False),
        ),
        Item('plot',
          show_label=False,
          editor=ComponentEditor(bgcolor=(0.8,0.8,0.8))),
      )
    )
  )

  def _zoomall_button_fired(self):
    self.plot.index_range.low_setting = 'auto'
    self.plot.index_range.high_setting = 'auto'
    self.plot.value_range.low_setting = 'auto'
    self.plot.value_range.high_setting = 'auto'

  def _center_button_fired(self):
    self.position_centered = not self.position_centered

  def _paused_button_fired(self):
    self.running = not self.running

  def _clear_button_fired(self):
    self.lats = []
    self.lngs = []
    self.alts = []
    self.lats_psuedo_abs = []
    self.lngs_psuedo_abs = []
    self.alts_psuedo_abs = []
    self.plot_data.set_data('lat', [])
    self.plot_data.set_data('lng', [])
    self.plot_data.set_data('alt', [])
    self.plot_data.set_data('t', [])
    self.plot_data.set_data('lat_ps', [])
    self.plot_data.set_data('lng_ps', [])
    self.plot_data.set_data('alt_ps', [])
    self.plot_data.set_data('t_ps', [])

  def _pos_llh_callback(self, sbp_msg):
    # Updating an ArrayPlotData isn't thread safe (see chaco issue #9), so
    # actually perform the update in the UI thread.
    if self.running:
      GUI.invoke_later(self.pos_llh_callback, sbp_msg)

  def update_table(self):
    self._table_list = self.table_spp.items()

  def pos_llh_callback(self, sbp_msg):
    soln = MsgPosLLH(sbp_msg)
    masked_flag = soln.flags & 0x7
    if masked_flag == 0:
      psuedo_absolutes = False
    else:
      psuedo_absolutes = True
    pos_table = []

    if self.log_file is None:
      self.log_file = open(time.strftime("position_log_%Y%m%d-%H%M%S.csv"), 'w')
    tow = soln.tow * 1e-3
    if self.nsec is not None:
      tow += self.nsec * 1e-9

    if self.week is not None:
      t = datetime.datetime(1980, 1, 6) + \
          datetime.timedelta(weeks=self.week) + \
          datetime.timedelta(seconds=tow)
      pos_table.append(('GPS Time', t))
      pos_table.append(('GPS Week', str(self.week)))

      self.log_file.write('%s,%.10f,%.10f,%.4f,%d,%d\n' % (
        str(t),
        soln.lat, soln.lon, soln.height,
        soln.n_sats, soln.flags)
      )
      self.log_file.flush()

    pos_table.append(('GPS ToW', tow))

    pos_table.append(('Num. sats', soln.n_sats))

    pos_table.append(('Lat', soln.lat))
    pos_table.append(('Lng', soln.lon))
    pos_table.append(('Alt', soln.height))
    pos_table.append(('Flags', '0x%02x' % soln.flags))
    if (soln.flags & 0xff) == 0:
      pos_table.append(('Mode', 'SPP (single point position)'))
    elif (soln.flags & 0xff) == 1:
      pos_table.append(('Mode', 'Fixed RTK'))
    elif (soln.flags & 0xff) == 2:
      pos_table.append(('Mode', 'Float RTK'))
    else:
      pos_table.append(('Mode', 'Unknown'))

    if psuedo_absolutes:
      # setup_plot variables
      self.lats_psuedo_abs.append(soln.lat)
      self.lngs_psuedo_abs.append(soln.lon)
      self.alts_psuedo_abs.append(soln.height)

      self.lats_psuedo_abs = self.lats_psuedo_abs[-1000:]
      self.lngs_psuedo_abs = self.lngs_psuedo_abs[-1000:]
      self.alts_psuedo_abs = self.alts_psuedo_abs[-1000:]

      self.plot_data.set_data('lat_ps', self.lats_psuedo_abs)
      self.plot_data.set_data('lng_ps', self.lngs_psuedo_abs)
      self.plot_data.set_data('alt_ps', self.alts_psuedo_abs)
      self.plot_data.set_data('cur_lat_ps', [soln.lat])
      self.plot_data.set_data('cur_lng_ps', [soln.lon])
      t_psuedo_abs = range(len(self.lats))
      self.plot_data.set_data('t', t)
      self.plot_data.set_data('t_ps', t_psuedo_abs)
      # set-up table variables
      self.table_psuedo_abs = pos_table

    else:
      # setup_plot variables
      self.lats.append(soln.lat)
      self.lngs.append(soln.lon)
      self.alts.append(soln.height)

      self.lats = self.lats[-1000:]
      self.lngs = self.lngs[-1000:]
      self.alts = self.alts[-1000:]

      self.plot_data.set_data('lat', self.lats)
      self.plot_data.set_data('lng', self.lngs)
      self.plot_data.set_data('alt', self.alts)
      self.plot_data.set_data('cur_lat', [soln.lat])
      self.plot_data.set_data('cur_lng', [soln.lon])
      t = range(len(self.lats))
      self.plot_data.set_data('t', t)

      # set-up table variables
      self.pos_table_spp = pos_table
      self.table_spp = self.pos_table_spp + self.vel_table + self.dops_table
      # TODO: figure out how to center the graph now that we have two separate messages
      # when we selectivtely send only SPP, the centering function won't work anymore
      if self.position_centered:
        d = (self.plot.index_range.high - self.plot.index_range.low) / 2.
        self.plot.index_range.set_bounds(soln.lon - d, soln.lon + d)
        d = (self.plot.value_range.high - self.plot.value_range.low) / 2.
        self.plot.value_range.set_bounds(soln.lat - d, soln.lat + d)


  def dops_callback(self, sbp_msg):
    dops = MsgDops(sbp_msg)
    self.dops_table = [
      ('PDOP', '%.1f' % (dops.pdop * 0.01)),
      ('GDOP', '%.1f' % (dops.gdop * 0.01)),
      ('TDOP', '%.1f' % (dops.tdop * 0.01)),
      ('HDOP', '%.1f' % (dops.hdop * 0.01)),
      ('VDOP', '%.1f' % (dops.vdop * 0.01))
    ]
    self.table_spp = self.pos_table_spp + self.vel_table + self.dops_table

  def vel_ned_callback(self, sbp_msg):
    vel_ned = MsgVelNED(sbp_msg)

    if self.vel_log_file is None:
      self.vel_log_file = open(time.strftime("velocity_log_%Y%m%d-%H%M%S.csv"), 'w')

    tow = vel_ned.tow * 1e-3
    if self.nsec is not None:
      tow += self.nsec * 1e-9

    if self.week is not None:
      t = datetime.datetime(1980, 1, 6) + \
          datetime.timedelta(weeks=self.week) + \
          datetime.timedelta(seconds=tow)

      self.vel_log_file.write('%s,%.6f,%.6f,%.6f,%.6f,%d\n' % (
        str(t),
        vel_ned.n * 1e-3, vel_ned.e * 1e-3, vel_ned.d * 1e-3,
        math.sqrt(vel_ned.n*vel_ned.n + vel_ned.e*vel_ned.e) * 1e-3,
        vel_ned.n_sats)
      )
      self.vel_log_file.flush()

    self.vel_table = [
      ('Vel. N', '% 8.4f' % (vel_ned.n * 1e-3)),
      ('Vel. E', '% 8.4f' % (vel_ned.e * 1e-3)),
      ('Vel. D', '% 8.4f' % (vel_ned.d * 1e-3)),
    ]
    self.table_spp = self.pos_table_spp + self.vel_table + self.dops_table

  def gps_time_callback(self, sbp_msg):
    self.week = MsgGPSTime(sbp_msg).wn
    self.nsec = MsgGPSTime(sbp_msg).ns

  def __init__(self, link):
    super(SolutionView, self).__init__()

    self.log_file = None
    self.vel_log_file = None

    self.plot_data = ArrayPlotData(lat=[], lng=[], alt=[], t=[],
      cur_lat=[], cur_lng=[], cur_lat_ps=[], cur_lng_ps=[],
      lat_ps=[], lng_ps=[], alt_ps=[], t_ps=[])
    self.plot = Plot(self.plot_data)

    # 1000 point buffer
    self.plot.plot(('lng', 'lat'), type='line',  name='', color=(0, 0, 0.9, 0.1))
    self.plot.plot(('lng', 'lat'), type='scatter',  name='',
      color='blue', marker='dot', line_width=0.0, marker_size=1.0)
    self.plot.plot(('lng_ps', 'lat_ps'), type='line',  name='', color=(1, 0.4, 0, 0.1))
    self.plot.plot(('lng_ps', 'lat_ps'), type='scatter', name='',
      color='orange', marker='diamond', line_width=0.0, marker_size=1.0)
    # current values
    spp = self.plot.plot(('cur_lng', 'cur_lat'), type='scatter', name='SPP',
      color='blue', marker='plus', line_width=1.5, marker_size=5.0)
    rtk = self.plot.plot(('cur_lng_ps', 'cur_lat_ps'), type='scatter',
      name='RTK', color='orange', marker='plus', line_width=1.5, marker_size=5.0)
    plot_labels = ['SPP','RTK']
    plots_legend = dict(zip(plot_labels, [spp,rtk]))
    self.plot.legend.plots = plots_legend
    self.plot.legend.visible = True



    self.plot.index_axis.tick_label_position = 'inside'
    self.plot.index_axis.tick_label_color = 'gray'
    self.plot.index_axis.tick_color = 'gray'
    self.plot.index_axis.title='Longitude (degrees)'
    self.plot.index_axis.title_spacing = 5
    self.plot.value_axis.tick_label_position = 'inside'
    self.plot.value_axis.tick_label_color = 'gray'
    self.plot.value_axis.tick_color = 'gray'
    self.plot.value_axis.title='Latitude (degrees)'
    self.plot.value_axis.title_spacing = 5
    self.plot.padding = (25, 25, 25, 25)

    self.plot.tools.append(PanTool(self.plot))
    zt = ZoomTool(self.plot, zoom_factor=1.1, tool_mode="box", always_on=False)
    self.plot.overlays.append(zt)

    self.link = link
    self.link.add_callback(self._pos_llh_callback, SBP_MSG_POS_LLH)
    self.link.add_callback(self.vel_ned_callback, SBP_MSG_VEL_NED)
    self.link.add_callback(self.dops_callback, SBP_MSG_DOPS)
    self.link.add_callback(self.gps_time_callback, SBP_MSG_GPS_TIME)

    self.week = None
    self.nsec = 0

    self.python_console_cmds = {
      'solution': self,
    }



#!/usr/bin/env python
# Copyright (C) 2014 Swift Navigation Inc.
# Contact: Gareth McMullin <gareth@swift-nav.com>
#
# This source is subject to the license found in the file 'LICENSE' which must
# be be distributed together with this source. All other rights reserved.
#
# THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
# EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.

import struct
import sys
import serial_link
import threading

from sbp.piksi import *

class FileIO(object):
  def __init__(self, link):
    self.link = link

  def read(self, filename):
    """
    Read the contents of a file.

    Parameters
    ----------
    filename : str
        Name of the file to read.

    Returns
    -------
    out : str
        Contents of the file.
    """
    chunksize = 255 - 6 - len(filename)
    buf = ''
    while True:
      msg = struct.pack("<IB", len(buf), chunksize) + filename + '\0'
      self.link.send(SBP_MSG_FILEIO_READ, msg)
      data = self.link.wait(SBP_MSG_FILEIO_READ, timeout=1.0)
      if not data:
        raise Exception("Timeout waiting for FILEIO_READ reply")
      if data[:len(msg)] != msg:
        raise Exception("Reply FILEIO_READ doesn't match request")
      chunk = data[len(msg):]
      buf += chunk
      if len(chunk) != chunksize:
        return buf

  def readdir(self, dirname='.'):
    """
    List the files in a directory.

    Parameters
    ----------
    dirname : str (optional)
        Name of the directory to list. Defaults to the root directory.

    Returns
    -------
    out : [str]
        List of file names.
    """
    files = []
    while True:
      msg = struct.pack("<I", len(files)) + dirname + '\0'
      self.link.send(SBP_MSG_FILEIO_READ_DIR, msg)
      data = self.link.wait(SBP_MSG_FILEIO_READ_DIR, timeout=1.0)
      if not data:
        raise Exception("Timeout waiting for FILEIO_READ_DIR reply")
      if data[:len(msg)] != msg:
        raise Exception("Reply FILEIO_READ_DIR doesn't match request")
      chunk = data[len(msg):].split('\0')
      files += chunk[:-1]
      if chunk[-1] == '\xff':
        return files

  def remove(self, filename):
    """
    Delete a file.

    Parameters
    ----------
    filename : str
        Name of the file to delete.
    """
    self.link.send(SBP_MSG_FILEIO_REMOVE, filename + '\0')

  def write(self, filename, data, offset=0, trunc=True):
    """
    Write to a file.

    Parameters
    ----------
    filename : str
        Name of the file to write to.
    data : str
        Data to write
    offset : int (optional)
        Offset into the file at which to start writing in bytes.
    trunc : bool (optional)
        Overwite the file, i.e. delete any existing file before writing. If
        this option is not specified and the existing file is longer than the
        current write then the contents of the file beyond the write will
        remain. If offset is non-zero then this flag is ignored.

    Returns
    -------
    out : str
        Contents of the file.
    """
    if trunc and offset == 0:
      self.remove(filename)
    chunksize = 255 - len(filename) - 5
    while data:
      chunk = data[:chunksize]
      data = data[chunksize:]
      header = struct.pack("<I", offset) + filename + '\0'
      self.link.send(SBP_MSG_FILEIO_WRITE, header + chunk)
      reply = self.link.wait(SBP_MSG_FILEIO_WRITE, timeout=1.0)
      if not reply:
        raise Exception("Timeout waiting for FILEIO_WRITE reply")
      if reply != header:
        raise Exception("Reply FILEIO_WRITE doesn't match request")
      offset += len(chunk)

def hexdump(data):
  """
  Print a hex dump.

  Parameters
  ----------
  data : indexable
      Data to display dump of, can be anything that supports length and index
      operations.
  """
  ret = ''
  ofs = 0
  while data:
    chunk = data[:16]
    data = data[16:]
    s = "%08X  " % ofs
    s += " ".join("%02X" % ord(c) for c in chunk[:8]) + "  "
    s += " ".join("%02X" % ord(c) for c in chunk[8:])
    s += "".join(" " for i in range(60 - len(s))) + "|"
    for c in chunk:
      s += c if 32 <= ord(c) < 128 else '.'
    s += '|\n'
    ofs += 16
    ret += s
  return ret

def print_dir_listing(files):
  """
  Print a directory listing.

  Parameters
  ----------
  files : [str]
      List of file names in the directory.
  """
  for f in files:
    print f

def get_args():
  """
  Get and parse arguments.
  """
  import argparse
  parser = argparse.ArgumentParser(description='Swift Nav File I/O Utility.')
  parser.add_argument('-r', '--read', nargs=1,
                     help='read a file')
  parser.add_argument('-l', '--list', default=None, nargs=1,
                     help='list a directory')
  parser.add_argument('-d', '--delete', nargs=1,
                     help='delete a file')
  parser.add_argument('-p', '--port',
                     default=[serial_link.SERIAL_PORT], nargs=1,
                     help='specify the serial port to use.')
  parser.add_argument("-b", "--baud",
                     default=[serial_link.SERIAL_BAUD], nargs=1,
                     help="specify the baud rate to use.")
  parser.add_argument("-v", "--verbose",
                     help="print extra debugging information.",
                     action="store_true")
  parser.add_argument("-x", "--hex",
                     help="output in hex dump format.",
                     action="store_true")
  parser.add_argument("-f", "--ftdi",
                     help="use pylibftdi instead of pyserial.",
                     action="store_true")
  return parser.parse_args()

def main():
  args = get_args()
  port = args.port[0]
  baud = args.baud[0]
  # Driver with context
  with serial_link.get_driver(args.ftdi, port, baud) as driver:
    # Handler with context
    with Handler(driver.read, driver.write, args.verbose) as link:
      f = FileIO(link)

      try:
        if args.read:
          data = f.read(args.read[0])
          if args.hex:
            print hexdump(data)
          else:
            print data
        elif args.delete:
          f.remove(args.delete[0])
        elif args.list is not None:
          print_dir_listing(f.readdir(args.list[0]))
        else:
          print "No command given, listing root directory:"
          print_dir_listing(f.readdir())
      except KeyboardInterrupt:
        pass

if __name__ == "__main__":
  main()



#!/usr/bin/env python
# Copyright (C) 2014 Swift Navigation Inc.
# Contact: Colin Beighley <colin@swift-nav.com>
#
# This source is subject to the license found in the file 'LICENSE' which must
# be be distributed together with this source. All other rights reserved.
#
# THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
# EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.

from urllib2 import urlopen, URLError
from json import load as jsonload
from urlparse import urlparse
import os

INDEX_URL = 'http://downloads.swiftnav.com/index.json'

class UpdateDownloader:

  def __init__(self):
    f = urlopen(INDEX_URL)
    self.index = jsonload(f)
    f.close()

  def download_stm_firmware(self):
    try:
      url = self.index['piksi_v2.3.1']['stm_fw']['url']
      filepath = self._download_file_from_url(url)
    except KeyError:
      raise KeyError("Error downloading firmware: URL not present in index")
    except URLError:
      raise URLError("Error: Failed to download latest NAP firmware from Swift Navigation's website")
    return filepath

  def download_nap_firmware(self):
    try:
      url = self.index['piksi_v2.3.1']['nap_fw']['url']
      filepath = self._download_file_from_url(url)
    except KeyError:
      raise KeyError("Error downloading firmware: URL not present in index")
    except URLError:
      raise URLError("Error: Failed to download latest NAP firmware from Swift Navigation's website")
    return filepath

  def _download_file_from_url(self, url):
    url = url.encode('ascii')
    urlpath = urlparse(url).path
    filename = os.path.split(urlparse(url).path)[1]

    url_file = urlopen(url)
    lines = url_file.readlines()
    with open(filename, 'w') as f:
      for line in lines:
        f.write(line)
    url_file.close()

    return os.path.abspath(filename)



import yaml
import pprint

class SettingsList():
  list_of_dicts = list();

  def __str__(self):
    pp = pprint.PrettyPrinter(indent=2)
    pp.pprint(self.list_of_dicts)

  def get_dict(self, group, name):
    for element in self.list_of_dicts:
        if element['name'] == name and element['group'] == group:
          return element

  def get_field(self, group, name,field):
    thisdict = self.get_dict(group, name)
    returnvar = ""
    if thisdict and isinstance(thisdict, dict):
      returnvar = thisdict.get(field,"")
    else:
      print "Error in settings list parsed yaml file."
      print "No entry for name  {0} and group is {1}".format(name,group)
    if not returnvar:
      returnvar = ""
    return returnvar

  def return_groups(self):
    group_set = set()
    output = []
    for element in self.list_of_dicts:
        group = element['group']
        if group not in group_set:
          output.append(element['group'])
          group_set.add(group)
    return output


  def __init__(self,path_to_file):
    stram = open(path_to_file, "r")
    self.list_of_dicts = yaml.load(stram)
    print "Loaded settings yaml file from path " + path_to_file
    print "Number of settings loaded {0}".format(len(self.list_of_dicts))




#!/usr/bin/env python
# Copyright (C) 2011-2014 Swift Navigation Inc.
# Contact: Fergus Noble <fergus@swift-nav.com>
#
# This source is subject to the license found in the file 'LICENSE' which must
# be be distributed together with this source. All other rights reserved.
#
# THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
# EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.

from traits.api import Str, Instance, Dict, HasTraits, Array, Float, Bool, on_trait_change, List, Int
from traitsui.api import Item, View, HGroup, ArrayEditor, HSplit

from chaco.api import BarPlot, ArrayDataSource, DataRange1D, LinearMapper, OverlayPlotContainer, LabelAxis, PlotAxis, ArrayPlotData, Plot, Legend
from chaco.tools.api import LegendTool
from enable.api import ComponentEditor, Component

from pyface.api import GUI

import struct

import numpy as np

from sbp.tracking import SBP_MSG_TRACKING_STATE

TRACKING_STATE_BYTES_PER_CHANNEL = 6
NUM_POINTS = 200

colours_list = [
  0xE41A1C,
  0x377EB8,
  0x4DAF4A,
  0x984EA3,
  0xFF7F00,
  0xFFFF33,
  0xA65628,
  0xF781BF,
]

class TrackingState(HasTraits):
  state = Int()
  cn0 = Float()
  prn = Int()

  def __init__(self, *args, **kwargs):
    self.update(*args, **kwargs)

  def update(self, state, prn, cn0):
    self.state = state
    self.cn0 = 0 if cn0 == -1 else cn0
    self.prn = prn

  def __repr__(self):
    return "TS: %d %f" % (self.prn, self.cn0)

class TrackingView(HasTraits):
  python_console_cmds = Dict()
  states = List(Instance(TrackingState))
  cn0_history = List()

  plot = Instance(Plot)
  plots = List()
  plot_data = Instance(ArrayPlotData)

  traits_view = View(
    HSplit(
      Item(
        'plot',
        editor = ComponentEditor(bgcolor = (0.8, 0.8, 0.8)),
        show_label = False,
      )
    )
  )

  def tracking_state_callback(self, sbp_msg):
    n_channels = len(sbp_msg.payload) / TRACKING_STATE_BYTES_PER_CHANNEL
    if n_channels != self.n_channels:
      # Update number of channels
      self.n_channels = n_channels
      self.states = [TrackingState(0, 0, 0) for _ in range(n_channels)]
      for pl in self.plot.plots.iterkeys():
        self.plot.delplot(pl.name)
      self.plots = []
      for n in range(n_channels):
        self.plot_data.set_data('ch'+str(n), [0.0])
        pl = self.plot.plot(('t', 'ch'+str(n)), type='line', color='auto', name='ch'+str(n))
        self.plots.append(pl)
      print 'Number of tracking channels changed to', n_channels

    fmt = '<' + n_channels * 'BBf'
    state_data = struct.unpack(fmt, sbp_msg.payload)
    for n, s in enumerate(self.states):
      s.update(*state_data[3*n:3*(n+1)])
    GUI.invoke_later(self.update_plot)

  def update_plot(self):
    self.cn0_history.append([s.cn0 for s in self.states])
    self.cn0_history = self.cn0_history[-1000:]

    chans = np.transpose(self.cn0_history[-NUM_POINTS:])
    plot_labels = []
    for n in range(self.n_channels):
      self.plot_data.set_data('ch'+str(n), chans[n])
      if self.states[n].state == 0:
        plot_labels.append('Ch %02d (Disabled)' % n)
      else:
        plot_labels.append('Ch %02d (PRN%02d)' % (n, self.states[n].prn+1))
    plots = dict(zip(plot_labels, self.plots))
    self.plot.legend.plots = plots

  def __init__(self, link):
    super(TrackingView, self).__init__()

    self.n_channels = None

    self.plot_data = ArrayPlotData(t=[0.0])
    self.plot = Plot(self.plot_data, auto_colors=colours_list, emphasized=True)
    self.plot.title = 'Tracking C/N0'
    self.plot.title_color = [0,0,0.43]
    self.plot.value_range.margin = 0.1
    self.plot.value_range.bounds_func = lambda l, h, m, tb: (0, h*(1+m))
    self.plot.value_axis.orientation = 'right'
    self.plot.value_axis.axis_line_visible = False
    self.plot.value_axis.title = 'dB-Hz'
    t = range(NUM_POINTS)
    self.plot_data.set_data('t', t)

    self.plot.legend.visible = True
    self.plot.legend.align = 'ul'
    self.plot.legend.tools.append(LegendTool(self.plot.legend, drag_button="right"))

    self.link = link
    self.link.add_callback(self.tracking_state_callback, SBP_MSG_TRACKING_STATE)

    self.python_console_cmds = {
      'track': self
    }



#!/usr/bin/env python
# Copyright (C) 2011-2015 Swift Navigation Inc.
# Contact: Fergus Noble <fergus@swift-nav.com>
#
# This source is subject to the license found in the file 'LICENSE' which must
# be be distributed together with this source. All other rights reserved.
#
# THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
# EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.

"""
The :mod:`piksi_tools.serial_link` module contains functions related to
setting up and running SBP message handling.
"""

import sys
import time

from sbp.piksi                          import SBP_MSG_PRINT, SBP_MSG_RESET
from sbp.client.drivers.file_driver     import FileDriver
from sbp.client.drivers.pyserial_driver import PySerialDriver
from sbp.client.drivers.pyftdi_driver   import PyFTDIDriver
from sbp.client.loggers.byte_logger     import ByteLogger
from sbp.client.loggers.json_logger     import JSONLogger
from sbp.client.loggers.null_logger     import NullLogger
from sbp.client.loggers.pickle_logger   import PickleLogger
from sbp.client.handler                 import Handler

LOG_FILENAME = time.strftime("serial-link-%Y%m%d-%H%M%S.log")

SERIAL_PORT  = "/dev/ttyUSB0"
SERIAL_BAUD  = 1000000

def get_ports():
  """
  Get list of serial ports.
  """
  import serial.tools.list_ports
  return [p for p in serial.tools.list_ports.comports() if p[1][0:4] != "ttyS"]

def get_args():
  """
  Get and parse arguments.
  """
  import argparse
  parser = argparse.ArgumentParser(description="Swift Navigation SBP Client.")
  parser.add_argument("-p", "--port",
                      default=[SERIAL_PORT], nargs=1,
                      help="specify the serial port to use.")
  parser.add_argument("-b", "--baud",
                      default=[SERIAL_BAUD], nargs=1,
                      help="specify the baud rate to use.")
  parser.add_argument("-v", "--verbose",
                      action="store_true",
                      help="print extra debugging information.")
  parser.add_argument("-f", "--ftdi",
                      action="store_true",
                      help="use pylibftdi instead of pyserial.")
  parser.add_argument("-l", "--log",
                      action="store_true",
                      help="serialize SBP messages to autogenerated log file.")
  parser.add_argument("-j", "--json",
                      action="store_true",
                      help="JSON serialize SBP messages.")
  parser.add_argument("-t", "--timeout",
                      default=[None], nargs=1,
                      help="exit after TIMEOUT seconds have elapsed.")
  parser.add_argument("-r", "--reset",
                      action="store_true",
                      help="reset device after connection.")
  parser.add_argument("-i", "--input-filename",
                      default=[None], nargs=1,
                      help="use input file to read SBP messages from.")
  parser.add_argument("-o", "--log-filename",
                      default=[LOG_FILENAME], nargs=1,
                      help="file to log output to.")
  parser.add_argument("-s", "--byte",
                      action="store_true",
                      help="Serialize SBP messages to bytes.")
  return parser.parse_args()

def get_driver(use_ftdi, port, baud, filename=None):
  """
  Get a driver based on configuration options

  Parameters
  ----------
  use_ftdi : bool
    For serial driver, use the pyftdi driver, otherwise use the pyserial driver.
  port : string
    Serial port to read.
  baud : int
    Serial port baud rate to set.
  filename : string
    File to read SBP messages from.
  """
  if filename:
    return FileDriver(filename)
  if use_ftdi:
    return PyFTDIDriver(baud)
  return PySerialDriver(port, baud)

def get_logger(use_log, use_json, use_byte, filename):
  """
  Get a logger based on configuration options.

  Parameters
  ----------
  use_log : bool
    Whether to log or not.
  use_json : bool
    Log out JSON records, otherwise log out bytes or pickled records.
  use_byte : bool
    Log out bytes, otherwise log out pickled records.
  filename : string
    File to log to.
  """
  if not use_log:
    return NullLogger()
  print "Logging at %s." % filename
  if use_json:
    return JSONLogger(filename)
  if use_byte:
    return ByteLogger(filename)
  return PickleLogger(filename)

def printer(sbp_msg):
  """
  Default print callback

  Parameters
  ----------
  sbp_msg: SBP
    SBP Message to print out.
  """
  sys.stdout.write(sbp_msg.payload)

def main():
  """
  Get configuration, get driver, get logger, and build handler and start it.
  """
  args = get_args()
  port = args.port[0]
  baud = args.baud[0]
  verbose = args.verbose
  use_ftdi = args.ftdi
  use_log = args.log
  use_json = args.json
  use_byte = args.byte
  timeout = args.timeout[0]
  reset = args.reset
  input_filename = args.input_filename[0]
  log_filename = args.log_filename[0]
  # Driver with context
  with get_driver(use_ftdi, port, baud, input_filename) as driver:
    # Handler with context
    with Handler(driver.read, driver.write, verbose) as link:
      # Logger with context
      with get_logger(use_log, use_json, use_byte, log_filename) as logger:
        link.add_callback(printer, SBP_MSG_PRINT)
        link.add_callback(logger)
        link.start()
        if reset:
          link.send(SBP_MSG_RESET, "")
        try:
          if timeout is None:
            while True:
              time.sleep(0.1)
          else:
            time.sleep(float(timeout))
        except KeyboardInterrupt:
          pass

if __name__ == "__main__":
  main()




#!/usr/bin/env python
# Copyright (C) 2011-2014 Swift Navigation Inc.
# Contact: Fergus Noble <fergus@swift-nav.com>
#
# This source is subject to the license found in the file 'LICENSE' which must
# be be distributed together with this source. All other rights reserved.
#
# THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
# EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.

from mayavi import mlab
import math
import numpy as np
import data_real4 as d

#x_min = -600.0
#x_max = -400.0

#y_min = 0.0
#y_max = 1023.0

x_min = 0.0
x_max = 1000.0

y_min = 0.0
y_max = 1000.0

d.zs = np.array(d.zs)

d.zs = 500 * d.zs / np.max(d.zs)

print d.zs.shape

x_len, y_len = d.zs.shape
xs = np.arange(x_min, x_max, (x_max-x_min)/x_len)
ys = np.arange(y_min, y_max, (y_max-y_min)/y_len)
xs, ys = np.meshgrid(ys, xs)

s = mlab.surf(xs, ys, d.zs)
mlab.show()



#!/usr/bin/env python
# Copyright (C) 2011-2014 Swift Navigation Inc.
# Contact: Fergus Noble <fergus@swift-nav.com>
#
# This source is subject to the license found in the file 'LICENSE' which must
# be be distributed together with this source. All other rights reserved.
#
# THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
# EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.

from traits.api import Instance, Dict, HasTraits, Array, Float, on_trait_change, List, Int, Button, Bool, Str
from traitsui.api import Item, View, HGroup, VGroup, ArrayEditor, HSplit, TabularEditor, VSplit
from traitsui.tabular_adapter import TabularAdapter
from chaco.api import ArrayPlotData, Plot
from chaco.tools.api import ZoomTool, PanTool
from enable.api import ComponentEditor
from enable.savage.trait_defs.ui.svg_button import SVGButton
from pyface.api import GUI

import struct
import math
import os
import numpy as np
import datetime

from sbp.observation import SBP_MSG_OBS
from sbp.tracking    import SBP_MSG_EPHEMERIS

class SimpleAdapter(TabularAdapter):
    columns = [('PRN', 0), ('Pseudorange',  1), ('Carrier Phase',  2), ('C/N0', 3)]

class Observation:
  def from_binary(self, data):
    soln = struct.unpack('<3d3d3d3d7ddddHBB', data)
    self.pos_llh = [soln[0]*(180/math.pi), soln[1]*(180/math.pi), soln[2]]
    self.pos_ecef = soln[3:6]
    self.vel_ned = soln[6:9]
    self.vel_ecef = soln[9:12]
    self.err_cov = soln[12:19]
    self.clock_offset = soln[19]
    self.clock_bias = soln[20]
    self.gps_tow = soln[21]
    self.gps_week = soln[22]
    self.soln_valid = soln[23]
    self.n_used = soln[24]


class ObservationView(HasTraits):
  python_console_cmds = Dict()

  _obs_table_list = List()
  obs = Dict()

  name = Str('Rover')

  recording = Bool(False)

  record_button = SVGButton(
    label='Record', tooltip='Record Raw Observations',
    toggle_tooltip='Stop Recording', toggle=True,
    filename=os.path.join(os.path.dirname(__file__), 'images', 'fontawesome', 'floppy-o.svg'),
    toggle_filename=os.path.join(os.path.dirname(__file__), 'images', 'fontawesome', 'stop.svg'),
    width=16, height=16
  )

  def trait_view(self, view):
    return View(
      HGroup(
        Item('_obs_table_list', style = 'readonly', editor = TabularEditor(adapter=SimpleAdapter()), show_label=False),
        VGroup(
          Item('record_button', show_label=False),
        ),
        label = self.name,
        show_border = True
      )
    )

  def _record_button_fired(self):
    self.recording = not self.recording
    if not self.recording:
      if self.rinex_file is not None:
        self.rinex_file.close()
      self.rinex_file = None

  def rinex_save(self):
    if self.recording:
      if self.rinex_file is None:
        # If the file is being opened for the first time, write the RINEX header
        self.rinex_file = open(self.name+self.t.strftime("-%Y%m%d-%H%M%S.obs"),  'w')
        header = """     2.11           OBSERVATION DATA    G (GPS)             RINEX VERSION / TYPE
pyNEX                                   %s UTC PGM / RUN BY / DATE
                                                            MARKER NAME
                                                            OBSERVER / AGENCY
                                                            REC # / TYPE / VERS
                                                            ANT # / TYPE
   808673.9171 -4086658.5368  4115497.9775                  APPROX POSITION XYZ
        0.0000        0.0000        0.0000                  ANTENNA: DELTA H/E/N
     1     0                                                WAVELENGTH FACT L1/2
     3    C1    L1    S1                                    # / TYPES OF OBSERV
%s%13.7f     GPS         TIME OF FIRST OBS
                                                            END OF HEADER
""" % (
            datetime.datetime.utcnow().strftime("%Y%m%d %H%M%S"),
            self.t.strftime("  %Y    %m    %d    %H    %M"), self.t.second + self.t.microsecond * 1e-6,
        )
        self.rinex_file.write(header)

      prns = list(self.obs.iterkeys())
      self.rinex_file.write("%s %10.7f  0 %2d" % (self.t.strftime(" %y %m %d %H %M"),
                                                  self.t.second + self.t.microsecond*1e-6,
                                                  len(prns)))
      while len(prns) > 0:
          prns_ = prns[:12]
          prns = prns[12:]
          for prn in prns_:
              self.rinex_file.write('G%2d' % (prn+1))
          self.rinex_file.write('   ' * (12 - len(prns_)))
          self.rinex_file.write('\n')

      for prn in list(self.obs.iterkeys()):
          # G    3 C1C L1C D1C
          self.rinex_file.write("%14.3f  " % self.obs[prn][0])
          self.rinex_file.write("%14.3f  " % self.obs[prn][1])
          self.rinex_file.write("%14.3f  \n" % self.obs[prn][2])

      self.rinex_file.flush()

  def update_obs(self):
    self._obs_table_list = [(prn + 1,) + obs for prn, obs in sorted(self.obs.items(), key=lambda x: x[0])]

  def obs_packed_callback(self, sbp_msg):
    if (sbp_msg.sender is not None and
        (self.relay ^ (sbp_msg.sender == 0))):
      return

    hdr_fmt = "<IHB"
    hdr_size = struct.calcsize(hdr_fmt)
    tow, wn, seq = struct.unpack(hdr_fmt, sbp_msg.payload[:hdr_size])

    tow = float(tow) / 1000.0

    total = seq >> 4
    count = seq & ((1 << 4) - 1)

    obs_fmt = '<IiBBHB'
    obs_size = struct.calcsize(obs_fmt)
    n_obs = (len(sbp_msg.payload) - hdr_size) / obs_size
    obs_data = sbp_msg.payload[hdr_size:]

    # Confirm this packet is good.
    # Assumes no out-of-order packets
    if (count == 0):
      self.gps_tow = tow;
      self.gps_week = wn;
      self.prev_obs_total = total
      self.prev_obs_count = 0;
      self.obs = {}

    elif (self.gps_tow            != tow    or
          self.gps_week           != wn     or
          self.prev_obs_count + 1 != count  or
          self.prev_obs_total     != total):
      print "We dropped a packet. Skipping this observation sequence"
      self.prev_obs_count = -1;
      return;

    else:
      self.prev_obs_count = count

    # Save this packet
    # See sbp_piksi.h for format
    for i in range(n_obs):
      P, Li, Lf, snr, lock, prn = struct.unpack(obs_fmt, obs_data[:obs_size])
      obs_data = obs_data[obs_size:]
      self.obs[prn] = (
        float(P) / 1e2,
        float(Li) + float(Lf) / (1<<8),
        float(snr) / 4)

    if (count == total - 1):
      self.t = datetime.datetime(1980, 1, 6) + \
               datetime.timedelta(weeks=self.gps_week) + \
               datetime.timedelta(seconds=self.gps_tow)

      self.update_obs()
      self.rinex_save()

    return

  def ephemeris_callback(self, sbp_msg):
    gps_time_fmt = "dH"
    eph_fmt = "<" + "d"*19 + gps_time_fmt*2 + "BBB"
    eph_size = struct.calcsize(eph_fmt)
    tgd, \
    crs, crc, cuc, cus, cic, cis, \
    dn, m0, ecc, sqrta, omega0, omegadot, w, inc, inc_dot, \
    af0, af1, af2, \
    toe_tow, toe_wn, toc_tow, toc_wn, \
    valid, \
    healthy, \
    prn = struct.unpack(eph_fmt, sbp_msg.payload[:eph_size])
    if self.recording:
      if self.eph_file is None:
        self.eph_file = open(self.name+self.t.strftime("-%Y%m%d-%H%M%S.eph"),  'w')
        header = "time, " \
               + "tgd, " \
               + "crs, crc, cuc, cus, cic, cis, " \
               + "dn, m0, ecc, sqrta, omega0, omegadot, w, inc, inc_dot, " \
               + "af0, af1, af2, " \
               + "toe_tow, toe_wn, toc_tow, toc_wn, " \
               + "valid, " \
               + "healthy, " \
               + "prn\n"
        self.eph_file.write(header)

      strout = "%s %10.7f" % (self.t.strftime(" %y %m %d %H %M"),
                              self.t.second + self.t.microsecond*1e-6)
      strout += "," + str([tgd, \
                           crs, crc, cuc, cus, cic, cis, \
                           dn, m0, ecc, sqrta, omega0, omegadot, w, inc, inc_dot, \
                           af0, af1, af2, \
                           toe_tow, toe_wn, toc_tow, toc_wn, \
                           valid, \
                           healthy, \
                           prn+1])[1: -1] + "\n"
      self.eph_file.write(strout)
      self.eph_file.flush()


  def __init__(self, link, name='Rover', relay=False):
    super(ObservationView, self).__init__()

    self.obs_count = 0

    self.gps_tow  = 0.0
    self.gps_week = 0

    self.relay = relay
    self.name  = name

    self.rinex_file = None
    self.eph_file   = None

    self.link = link
    self.link.add_callback(self.obs_packed_callback, SBP_MSG_OBS)
    self.link.add_callback(self.ephemeris_callback, SBP_MSG_EPHEMERIS)

    self.python_console_cmds = {
      'obs': self
    }



# Copyright (C) 2011-2014 Swift Navigation Inc.
# Contact: Fergus Noble <fergus@swift-nav.com>
#
# This source is subject to the license found in the file 'LICENSE' which must
# be be distributed together with this source. All other rights reserved.
#
# THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
# EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.

from traits.api import Instance, Dict, HasTraits, Array, Float, on_trait_change, List, Int, Button, Bool
from traitsui.api import Item, View, HGroup, VGroup, ArrayEditor, HSplit, TabularEditor
from traitsui.tabular_adapter import TabularAdapter
from chaco.api import ArrayPlotData, Plot
from chaco.tools.api import ZoomTool, PanTool
from enable.api import ComponentEditor
from enable.savage.trait_defs.ui.svg_button import SVGButton
from pyface.api import GUI

import struct
import math
import os
import numpy as np
import datetime
import time

from sbp.piksi      import *
from sbp.navigation import *

class SimpleAdapter(TabularAdapter):
    columns = [('Item', 0), ('Value',  1)]
    width = 80

class Baseline:
  def from_binary(self, data):
    soln = struct.unpack('<3ddHHB', data)
    self.ned = np.array([soln[0], soln[1], soln[2]])
    self.tow = soln[3] / 1e3
    self.wn = soln[4]
    self.flags = soln[5]
    self.n_sats = soln[6]

class BaselineView(HasTraits):
  python_console_cmds = Dict()

  ns = List()
  es = List()
  ds = List()

  table = List()

  plot = Instance(Plot)
  plot_data = Instance(ArrayPlotData)

  running = Bool(True)
  position_centered = Bool(False)

  clear_button = SVGButton(
    label='', tooltip='Clear',
    filename=os.path.join(os.path.dirname(__file__), 'images', 'iconic', 'x.svg'),
    width=16, height=16
  )
  zoomall_button = SVGButton(
    label='', tooltip='Zoom All',
    filename=os.path.join(os.path.dirname(__file__), 'images', 'iconic', 'fullscreen.svg'),
    width=16, height=16
  )
  center_button = SVGButton(
    label='', tooltip='Center on Baseline', toggle=True,
    filename=os.path.join(os.path.dirname(__file__), 'images', 'iconic', 'target.svg'),
    width=16, height=16
  )
  paused_button = SVGButton(
    label='', tooltip='Pause', toggle_tooltip='Run', toggle=True,
    filename=os.path.join(os.path.dirname(__file__), 'images', 'iconic', 'pause.svg'),
    toggle_filename=os.path.join(os.path.dirname(__file__), 'images', 'iconic', 'play.svg'),
    width=16, height=16
  )

  reset_button = Button(label='Reset Filters')
  reset_iar_button = Button(label='Reset IAR')
  init_base_button = Button(label='Init. with known baseline')

  traits_view = View(
    HSplit(
      Item('table', style = 'readonly', editor = TabularEditor(adapter=SimpleAdapter()), show_label=False, width=0.3),
      VGroup(
        HGroup(
          Item('paused_button', show_label=False),
          Item('clear_button', show_label=False),
          Item('zoomall_button', show_label=False),
          Item('center_button', show_label=False),
          Item('reset_button', show_label=False),
          Item('reset_iar_button', show_label=False),
          Item('init_base_button', show_label=False),
        ),
        Item(
          'plot',
          show_label = False,
          editor = ComponentEditor(bgcolor = (0.8,0.8,0.8)),
        )
      )
    )
  )

  def _zoomall_button_fired(self):
    self.plot.index_range.low_setting = 'auto'
    self.plot.index_range.high_setting = 'auto'
    self.plot.value_range.low_setting = 'auto'
    self.plot.value_range.high_setting = 'auto'

  def _center_button_fired(self):
    self.position_centered = not self.position_centered

  def _paused_button_fired(self):
    self.running = not self.running

  def _reset_button_fired(self):
    self.link.send(SBP_MSG_RESET_FILTERS, '\x00')

  def _reset_iar_button_fired(self):
    self.link.send(SBP_MSG_RESET_FILTERS, '\x01')

  def _init_base_button_fired(self):
    self.link.send(SBP_MSG_INIT_BASE, '')

  def _clear_button_fired(self):
    self.ns = []
    self.es = []
    self.ds = []
    self.plot_data.set_data('n', [])
    self.plot_data.set_data('e', [])
    self.plot_data.set_data('d', [])
    self.plot_data.set_data('t', [])
    self.plot_data.set_data('curr_n', [])
    self.plot_data.set_data('curr_e', [])
    self.plot_data.set_data('curr_d', [])

  def _baseline_callback_ecef(self, data):
    #Don't do anything for ECEF currently
    return

  def iar_state_callback(self, sbp_msg):
    self.num_hyps = struct.unpack('<I', sbp_msg.payload)[0]

  def _baseline_callback_ned(self, sbp_msg):
    # Updating an ArrayPlotData isn't thread safe (see chaco issue #9), so
    # actually perform the update in the UI thread.
    if self.running:
      GUI.invoke_later(self.baseline_callback, sbp_msg)

  def update_table(self):
    self._table_list = self.table.items()

  def gps_time_callback(self, sbp_msg):
    self.week = MsgGPSTime(sbp_msg).wn
    self.nsec = MsgGPSTime(sbp_msg).ns

  def baseline_callback(self, sbp_msg):
    soln = MsgBaselineNED(sbp_msg)
    table = []

    soln.n = soln.n * 1e-3
    soln.e = soln.e * 1e-3
    soln.d = soln.d * 1e-3

    dist = np.sqrt(soln.n**2 + soln.e**2 + soln.d**2)

    tow = soln.tow * 1e-3
    if self.nsec is not None:
      tow += self.nsec * 1e-9

    if self.week is not None:
      t = datetime.datetime(1980, 1, 6) + \
          datetime.timedelta(weeks=self.week) + \
          datetime.timedelta(seconds=tow)

      table.append(('GPS Time', t))
      table.append(('GPS Week', str(self.week)))

      if self.log_file is None:
        self.log_file = open(time.strftime("baseline_log_%Y%m%d-%H%M%S.csv"), 'w')

      self.log_file.write('%s,%.4f,%.4f,%.4f,%.4f,%d,0x%02x,%d\n' % (
        str(t),
        soln.n, soln.e, soln.d, dist,
        soln.n_sats,
        soln.flags,
        self.num_hyps)
      )
      self.log_file.flush()

    table.append(('GPS ToW', tow))

    table.append(('N', soln.n))
    table.append(('E', soln.e))
    table.append(('D', soln.d))
    table.append(('Dist.', dist))
    table.append(('Num. Sats.', soln.n_sats))
    table.append(('Flags', '0x%02x' % soln.flags))
    if soln.flags & 1:
      table.append(('Mode', 'Fixed RTK'))
    else:
      table.append(('Mode', 'Float'))
    table.append(('IAR Num. Hyps.', self.num_hyps))

    self.ns.append(soln.n)
    self.es.append(soln.e)
    self.ds.append(soln.d)

    self.ns = self.ns[-1000:]
    self.es = self.es[-1000:]
    self.ds = self.ds[-1000:]

    self.plot_data.set_data('n', self.ns)
    self.plot_data.set_data('e', self.es)
    self.plot_data.set_data('d', self.ds)
    self.plot_data.set_data('curr_n', [soln.n])
    self.plot_data.set_data('curr_e', [soln.e])
    self.plot_data.set_data('curr_d', [soln.d])
    self.plot_data.set_data('ref_n', [0.0])
    self.plot_data.set_data('ref_e', [0.0])
    self.plot_data.set_data('ref_d', [0.0])

    t = range(len(self.ns))
    self.plot_data.set_data('t', t)

    if self.position_centered:
      d = (self.plot.index_range.high - self.plot.index_range.low) / 2.
      self.plot.index_range.set_bounds(soln.e - d, soln.e + d)
      d = (self.plot.value_range.high - self.plot.value_range.low) / 2.
      self.plot.value_range.set_bounds(soln.n - d, soln.n + d)

    self.table = table

  def __init__(self, link):
    super(BaselineView, self).__init__()

    self.log_file = None

    self.num_hyps = 0

    self.plot_data = ArrayPlotData(n=[0.0], e=[0.0], d=[0.0], t=[0.0], ref_n=[0.0],
                     ref_e=[0.0], ref_d=[0.0], curr_e=[0.0], curr_n=[0.0], curr_d=[0.0])
    self.plot = Plot(self.plot_data)
    lin = self.plot.plot(('e', 'n'),
        type='line',
        color=(0, 0, 1, 0.1))
    pts = self.plot.plot(('e', 'n'),
        type='scatter',
        color='blue',
        marker='dot',
        line_width=0.0,
        marker_size=1.0)
    ref = self.plot.plot(('ref_e', 'ref_n'),
        type='scatter',
        color='red',
        marker='plus',
        marker_size=5,
        line_width=1.5)
    cur = self.plot.plot(('curr_e', 'curr_n'),
        type='scatter',
        color='orange',
        marker='plus',
        marker_size=5,
        line_width=1.5)
    plot_labels = ['Base Position','Rover Relative Position','Rover Path']
    plots_legend = dict(zip(plot_labels, [ref,cur,pts]))
    self.plot.legend.plots = plots_legend
    self.plot.legend.visible = True

    self.plot.index_axis.tick_label_position = 'inside'
    self.plot.index_axis.tick_label_color = 'gray'
    self.plot.index_axis.tick_color = 'gray'
    self.plot.index_axis.title='E (meters)'
    self.plot.index_axis.title_spacing = 5
    self.plot.value_axis.tick_label_position = 'inside'
    self.plot.value_axis.tick_label_color = 'gray'
    self.plot.value_axis.tick_color = 'gray'
    self.plot.value_axis.title='N (meters)'
    self.plot.value_axis.title_spacing = 5
    self.plot.padding = (25, 25, 25, 25)

    self.plot.tools.append(PanTool(self.plot))
    zt = ZoomTool(self.plot, zoom_factor=1.1, tool_mode="box", always_on=False)
    self.plot.overlays.append(zt)

    self.week = None
    self.nsec = 0

    self.link = link
    self.link.add_callback(self._baseline_callback_ned, SBP_MSG_BASELINE_NED)
    self.link.add_callback(self._baseline_callback_ecef, SBP_MSG_BASELINE_ECEF)
    self.link.add_callback(self.iar_state_callback, SBP_MSG_IAR_STATE)
    self.link.add_callback(self.gps_time_callback, SBP_MSG_GPS_TIME)

    self.python_console_cmds = {
      'baseline': self
    }



#!/usr/bin/env python
# Copyright (C) 2011-2014 Swift Navigation Inc.
# Contact: Fergus Noble <fergus@swift-nav.com>
#
# This source is subject to the license found in the file 'LICENSE' which must
# be be distributed together with this source. All other rights reserved.
#
# THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
# EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.

from traits.api import Instance, Dict, HasTraits, Array, Float, \
                       on_trait_change, List, Int, Button, Bool, Str, Color, \
                       Constant, Font, Undefined, Property, Any, Enum
from traitsui.api import Item, UItem, View, HGroup, Handler, VGroup, \
                         ArrayEditor, HSplit, TabularEditor, TextEditor, \
                         EnumEditor
from traitsui.tabular_adapter import TabularAdapter
from traits.etsconfig.api import ETSConfig

if ETSConfig.toolkit != 'null':
  from enable.savage.trait_defs.ui.svg_button import SVGButton
else:
  SVGButton = dict
from pyface.api import GUI

import struct
import math
import os
import numpy as np
import datetime


from fileio import FileIO
import callback_prompt as prompt

from sbp.piksi    import *
from sbp.standard import SBP_MSG_STARTUP

from settings_list import SettingsList


def u16_to_str(i):
  return chr(i & 0xff) + chr(i >> 8)

class SettingBase(HasTraits):
  name = Str()
  description = Str()
  notes = Str()
  value = Str(Undefined)
  ordering = Float(0)
  default_value = Str()

  traits_view = View()

  def __repr__(self):
    return "<Setting '%s' = '%s'>" % (self.name, self.value)

  def __str__(self):
    return self.value

class MyTextEditor(TextEditor):
  def init(self,parent):
    parent.read_only = True
    parent.multi_line = True

class Setting(SettingBase):
  full_name = Str()
  section = Str()

  traits_view = View(
    VGroup(
      Item('full_name', label='Name', style='readonly'),
      Item('value', editor=TextEditor(auto_set=False, enter_set=True)),
      Item('description', style='readonly'),
      Item('default_value', style='readonly'),
      UItem('notes', label="Notes", height=-1,
            editor=MyTextEditor(TextEditor(multi_line=True)), style='readonly',
            show_label=True, resizable=True),
      show_border=True,
      label='Setting',
    ),
  )

  def __init__(self, name, section, value, ordering, settings):
    self.name = name
    self.section = section
    self.full_name = "%s.%s" % (section, name)
    self.value = value
    self.ordering = ordering
    self.settings = settings
    self.description = settings.settings_yaml.get_field(section,
                                                           name, 'Description')
    self.notes = settings.settings_yaml.get_field(section, name, 'Notes')
    self.default_value = settings.settings_yaml.get_field(section, name,
                                                             'default value')

  def _value_changed(self, name, old, new):
    if (old != new and
        old is not Undefined and
        new is not Undefined):
      if type(self.value) == unicode:
        self.value = self.value.encode('ascii', 'replace')
      self.settings.set(self.section, self.name, self.value)

class EnumSetting(Setting):
  values = List()
  traits_view = View(
    VGroup(
      Item('full_name', label='Name', style='readonly'),
      Item('value', editor=EnumEditor(name='values')),
      Item('description', style='readonly'),
      Item('default_value', style='readonly'),
            UItem('notes', label="Notes", height=-1,
            editor=MyTextEditor(TextEditor(multi_line=True)), style='readonly',
            show_label=True, resizable=True),
      show_border=True,
      label='Setting',
    ),
  )

  def __init__(self, name, section, value, ordering, values, **kwargs):
    self.values = values
    Setting.__init__(self, name, section, value, ordering, **kwargs)

class SectionHeading(SettingBase):
  value = Constant('')

  def __init__(self, name):
    self.name = name

class SimpleAdapter(TabularAdapter):
  columns = [('Name', 'name'), ('Value',  'value')]
  font = Font('12')
  can_edit = Bool(False)
  SectionHeading_bg_color = Color(0xE0E0E0)
  SectionHeading_font = Font('14 bold')
  SectionHeading_name_text = Property
  Setting_name_text = Property
  name_width = Float(0.5)
  value_width = Float(0.9)

  def _get_SectionHeading_name_text(self):
    return self.item.name.replace('_', ' ')

  def _get_Setting_name_text(self):
    return self.item.name.replace('_', ' ')

class SettingsView(HasTraits):

  settings_yaml = list()

  settings_read_button = SVGButton(
    label='Reload', tooltip='Reload settings from Piksi',
    filename=os.path.join(os.path.dirname(__file__), 'images', 'fontawesome', 'refresh.svg'),
    width=16, height=16
  )

  settings_save_button = SVGButton(
    label='Save to Flash', tooltip='Save settings to Flash',
    filename=os.path.join(os.path.dirname(__file__), 'images', 'fontawesome', 'download.svg'),
    width=16, height=16
  )

  factory_default_button = SVGButton(
    label='Reset to Defaults', tooltip='Reset to Factory Defaults',
    filename=os.path.join(os.path.dirname(__file__), 'images', 'fontawesome', 'exclamation-triangle.svg'),
    width=16, height=16
  )

  settings_list = List(SettingBase)
  selected_setting = Instance(SettingBase)

  traits_view = View(
    HSplit(
      Item('settings_list',
        editor = TabularEditor(
          adapter=SimpleAdapter(),
          editable_labels=False,
          auto_update=True,
          selected='selected_setting'
        ),
        show_label=False,
      ),
      VGroup(
        HGroup(
          Item('settings_read_button', show_label=False),
          Item('settings_save_button', show_label=False),
          Item('factory_default_button', show_label=False),
        ),
        Item('selected_setting', style='custom', show_label=False),
      ),
    )
  )

  def _settings_read_button_fired(self):
    self.settings.clear()
    self.enumindex = 0
    self.ordering_counter = 0
    self.link.send(SBP_MSG_SETTINGS_READ_BY_INDEX, u16_to_str(self.enumindex))

  def _settings_save_button_fired(self):
    self.link.send(SBP_MSG_SETTINGS_SAVE, "")

  def _factory_default_button_fired(self):
    confirm_prompt = prompt.CallbackPrompt(
                          title="Reset to Factory Defaults?",
                          actions=[prompt.close_button, prompt.reset_button],
                          callback=self.reset_factory_defaults
                         )
    confirm_prompt.text = "This will erase all settings and then reset the device.\n" \
                        + "Are you sure you want to reset to factory defaults?"
    confirm_prompt.run(block=False)

  def reset_factory_defaults(self):
    # Delete settings file
    fio = FileIO(self.link)
    fio.remove('config')
    # Reset the Piksi
    self.link.send(SBP_MSG_RESET, '')

  ##Callbacks for receiving messages

  def settings_read_by_index_callback(self, sbp_msg):
    if not sbp_msg.payload:
      self.settings_list = []

      sections = sorted(self.settings.keys())

      for sec in sections:
        self.settings_list.append(SectionHeading(sec))
        for name, setting in sorted(self.settings[sec].iteritems(), key=lambda (n, s): s.ordering):
          self.settings_list.append(setting)

      for cb in self.read_finished_functions:
        if self.gui_mode:
          GUI.invoke_later(cb)
        else:
          cb()
      return

    section, setting, value, format_type = sbp_msg.payload[2:].split('\0')[:4]
    self.ordering_counter += 1

    if format_type == '':
      format_type = None
    else:
      setting_type, setting_format = format_type.split(':')

    if not self.settings.has_key(section):
      self.settings[section] = {}

    if format_type is None:
      # Plain old setting, no format information
      self.settings[section][setting] = Setting(setting, section, value,
                                                ordering=self.ordering_counter,
                                                settings=self
                                               )
    else:
      if setting_type == 'enum':
        enum_values = setting_format.split(',')
        self.settings[section][setting] = EnumSetting(setting, section, value,
                                                      ordering=self.ordering_counter,
                                                      values=enum_values,
                                                      settings=self
                                                     )
      else:
        # Unknown type, just treat is as a string
        self.settings[section][setting] = Setting(setting, section, value,
                                                  settings=self
                                                 )

    self.enumindex += 1
    self.link.send(SBP_MSG_SETTINGS_READ_BY_INDEX, u16_to_str(self.enumindex))

  def settings_read_callback(self, sbp_msg):
    section, setting, value = sbp_msg.payload.split('\0')[:3]
    # Hack to prevent an infinite loop of setting settings
    self.settings[section][setting].value = Undefined
    self.settings[section][setting].value = value

  def piksi_startup_callback(self, sbp_msg):
    self._settings_read_button_fired()

  def set(self, section, name, value):
      self.link.send(SBP_MSG_SETTINGS,
          '%s\0%s\0%s\0' % (section, name, value))

  def __init__(self, link, read_finished_functions=[], gui_mode=True):
    super(SettingsView, self).__init__()

    self.gui_mode = gui_mode
    self.enumindex = 0
    self.settings = {}
    self.link = link
    self.link.add_callback(self.settings_read_callback, SBP_MSG_SETTINGS)
    self.link.add_callback(self.piksi_startup_callback, SBP_MSG_STARTUP)
    self.link.add_callback(self.settings_read_by_index_callback,
      SBP_MSG_SETTINGS_READ_BY_INDEX)

    # Read in yaml file for setting metadatas (hardcoded filename for now)
    self.settings_yaml = SettingsList("settings.yaml")

    # List of functions to be executed after all settings are read.
    # No support for arguments currently.
    self.read_finished_functions = read_finished_functions

    self.setting_detail = SettingBase()

    self._settings_read_button_fired()

    self.python_console_cmds = {
      'settings': self
    }



#!/usr/bin/env python
# Copyright (C) 2014 Swift Navigation Inc.
# Contact: Colin Beighley <colin@swift-nav.com>
#
# This source is subject to the license found in the file 'LICENSE' which must
# be be distributed together with this source. All other rights reserved.
#
# THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
# EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.

from traits.api import HasTraits, Event, String, Button, Instance
from traitsui.api import View, Handler, Action, Item, TextEditor, VGroup, UItem
from pyface.api import GUI

from threading import Thread
from time import sleep

update_button = Action(name = "Update", action = "set_execute_callback_true", \
                             show_label=False)
reset_button = Action(name = "Reset", action = "set_execute_callback_true", \
                             show_label=False)
close_button = Action(name = "Close", action = "set_execute_callback_false", \
                             show_label=False)

from new import instancemethod

import sys, os
from pyface.image_resource import ImageResource
if getattr(sys, 'frozen', False):
    # we are running in a |PyInstaller| bundle
    basedir = sys._MEIPASS
    os.chdir(basedir)
else:
    # we are running in a normal Python environment
    basedir = os.path.dirname(__file__)
icon = ImageResource('icon',
         search_path=['images', os.path.join(basedir, 'images')])

# Handler methods that can be associated with buttons.
def set_execute_callback_true(self, info):
  info.object.execute_callback = True
  info.object.handler_executed = True

def set_execute_callback_false(self, info):
  info.object.execute_callback = False
  info.object.handler_executed = True

class CallbackHandler(Handler):

  # Register action handlers from passed list.
  def __init__(self, actions):
    super(CallbackHandler, self).__init__()
    for a in actions:
      # Add instancemethod to self for Action.action.
      handler = globals()[a.action]
      self.__dict__[a.action] = instancemethod(handler, self, CallbackHandler)

  # X button was pressed.
  def close(self, info, is_ok):
    info.object.handler_executed = True
    info.object.closed = True
    return True

  # Handles parent object's close Event being written to.
  def object_close_changed(self, info):
    info.object.closed = True
    info.ui.owner.close()

class CallbackPrompt(HasTraits):

  text = String
  close = Event

  def __init__(self, title, actions, callback=None):
    self.callback = callback

    self.handler_executed = False
    self.execute_callback = False
    self.closed = False
    self.close = 0

    self.view = View(
      Item(
        'text',
        style='readonly',
        editor=TextEditor(),
        show_label=False
      ),
      buttons=actions,
      title=title,
      handler=CallbackHandler(actions),
      icon = icon,
      resizable=True,
    )

  def run(self, block=True):
    try:
      if self.thread.is_alive():
        return
    except AttributeError:
      pass

    self.thread = Thread(target=self._run)
    self.thread.start()

    if block:
      self.wait()

  def _run(self):
    GUI.invoke_later(self.edit_traits, self.view)
    while not self.handler_executed:
      sleep(0.1)

    if self.execute_callback:
      GUI.invoke_later(self.callback)

    if not self.closed:
      self.close = 1

  def wait(self):
    while not self.closed:
      sleep(0.1)

  def kill(self):
    self.handler_executed = True
    if not self.closed:
      self.close = 1




#!/usr/bin/env python
# Copyright (C) 2011-2014 Swift Navigation Inc.
# Contact: Fergus Noble <fergus@swift-nav.com>
#
# This source is subject to the license found in the file 'LICENSE' which must
# be be distributed together with this source. All other rights reserved.
#
# THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
# EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.


# Auto-generate the simulation_data.c file from a given almanac.
#   Written by Niels Joubert
#   April 2014

from almanac import *


import argparse

def to_struct(sat):
  return "{ \n\
  .ecc       = %f,\n\
  .toa       = %f,\n\
  .inc       = %f,\n\
  .rora      = %f,\n\
  .a         = %f,\n\
  .raaw      = %f,\n\
  .argp      = %f,\n\
  .ma        = %f,\n\
  .af0       = %f,\n\
  .af1       = %f,\n\
  .week      = %d,\n\
  .prn       = %d,\n\
  .healthy   = %d,\n\
  .valid     = %d,\n\
}" % (sat.ecc,
  sat.toa,
  sat.inc,
  sat.rora,
  sat.a,
  sat.raaw,
  sat.argp,
  sat.ma,
  sat.af0,
  sat.af1,
  sat.week,
  sat.prn,
  sat.healthy,
  1);

if __name__ == "__main__":
  parser = argparse.ArgumentParser(description='Swift Nav Almanac C Generator')
  parser.add_argument("file",
    help="the almanac file to process into C structs")

  args = parser.parse_args();

  alm = Almanac()    
  with open(args.file) as f:
    alm.process_yuma(f.readlines())
    print "#include \"simulator_data.h\""
    print "/* AUTO-GENERATED FROM aimulator_almanac_generator.py */\n"
    print "u16 simulation_week_number = 1787;\n"
    print "double simulation_sats_pos[%d][3];\n" % len(alm.sats)
    print "double simulation_sats_vel[%d][3];\n" % len(alm.sats)
    print "u32 simulation_fake_carrier_bias[%d];\n" % len(alm.sats)
    print "u8 simulation_num_almanacs = %d;\n" % len(alm.sats)
    print "almanac_t simulation_almanacs[%d] = {" % len(alm.sats)
    for s in alm.sats:
      print "%s," % to_struct(s)
    print "};"



#!/usr/bin/env python
# Copyright (C) 2014 Swift Navigation Inc.
# Contact: Colin Beighley <colin@swift-nav.com>
#
# This source is subject to the license found in the file 'LICENSE' which must
# be be distributed together with this source. All other rights reserved.
#
# THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
# EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.

from urllib2 import URLError
from json import load as jsonload
from time import sleep
from intelhex import IntelHex, HexRecordError, HexReaderError
from pkg_resources import parse_version

from sbp.piksi import SBP_MSG_BOOTLOADER_JUMP_TO_APP, SBP_MSG_RESET

from threading import Thread

from traits.api import HasTraits, Event, String, Button, Instance, Int, Bool, \
                       on_trait_change
from traitsui.api import View, Handler, Action, Item, TextEditor, VGroup, \
                         UItem, InstanceEditor, VSplit, HSplit, HGroup, \
                         BooleanEditor
from pyface.api import GUI, FileDialog, OK, ProgressDialog

from piksi_tools import __version__ as CONSOLE_VERSION
import bootload
import flash
import callback_prompt as prompt
from update_downloader import UpdateDownloader
from output_stream import OutputStream

import sys, os
from pyface.image_resource import ImageResource
if getattr(sys, 'frozen', False):
    # we are running in a |PyInstaller| bundle
    basedir = sys._MEIPASS
    os.chdir(basedir)
else:
    # we are running in a normal Python environment
    basedir = os.path.dirname(__file__)
icon = ImageResource('icon',
         search_path=['images', os.path.join(basedir, 'images')])

INDEX_URL = 'http://downloads.swiftnav.com/index.json'


class IntelHexFileDialog(HasTraits):

  file_wildcard = String("Intel HEX File (*.hex)|*.hex|All files|*")

  status = String('Please choose a file')
  choose_fw = Button(label='Choose Firmware File')
  view = View(
               UItem('status'),
               UItem('choose_fw')
             )

  def __init__(self, flash_type):
    """
    Pop-up file dialog to choose an IntelHex file, with status and button to
    display in traitsui window.

    Parameters
    ----------
    flash_type : string
      Which Piksi flash to interact with ("M25" or "STM").
    """
    if not flash_type=='M25' and not flash_type=='STM':
      raise ValueError("flash_type must be 'M25' or 'STM'")
    self._flash_type = flash_type
    self.ihx = None

  def clear(self, status):
    """
    Set text of status box and clear IntelHex file.

    Parameters
    ----------
    status : string
      Error text to replace status box text with.
    """
    self.ihx = None
    self.status = status

  def load_ihx(self, filepath):
    """
    Load IntelHex file and set status to indicate if file was
    successfully loaded.

    Parameters
    ----------
    filepath : string
      Path to IntelHex file.
    """
    try:
      self.ihx = IntelHex(filepath)
      self.status = os.path.split(filepath)[1]
    except HexRecordError:
      self.clear('Error: File is not a valid Intel HEX File')

    # Check that address ranges are valid for self._flash_type.
    ihx_addrs = flash.ihx_ranges(self.ihx)
    if self._flash_type == "M25":
      try:
        sectors = flash.sectors_used(ihx_addrs, flash.m25_addr_sector_map)
      except IndexError:
        self.clear('Error: HEX File contains restricted address ' + \
                        '(STM Firmware File Chosen?)')
    elif self._flash_type == "STM":
      try:
        sectors = flash.sectors_used(ihx_addrs, flash.stm_addr_sector_map)
      except:
        self.clear('Error: HEX File contains restricted address ' + \
                        '(NAP Firmware File Chosen?)')

  def _choose_fw_fired(self):
    """ Activate file dialog window to choose IntelHex firmware file. """
    dialog = FileDialog(label='Choose Firmware File',
                        action='open', wildcard=self.file_wildcard)
    dialog.open()
    if dialog.return_code == OK:
      filepath = os.path.join(dialog.directory, dialog.filename)
      self.load_ihx(filepath)
    else:
      self.clear('Error while selecting file')

class PulsableProgressDialog(ProgressDialog):

  def __init__(self, max, pulsed=False):
    """
    Pop-up window for showing a process's progress.

    Parameters
    ----------
    max : int
      Maximum value of the progress bar.
    pulsed : bool
      Show non-partial progress initially.
    """
    super(PulsableProgressDialog, self).__init__()
    self.min = 0
    self.max = 0
    self.pulsed = pulsed
    self.passed_max = max

  def progress(self, count):
    """
    Update progress of progress bar. If pulsing initially, wait until count
    is at least 12 before changing to discrete progress bar.

    Parameters
    ----------
    count : int
      Current value of progress.
    """
    # Provide user feedback initially via pulse for slow sector erases.
    if self.pulsed:
      if count > 12:
        self.max = 100
        GUI.invoke_later(self.update, int(100*float(count)/self.passed_max))
    else:
      self.max = 100
      GUI.invoke_later(self.update, int(100*float(count)/self.passed_max))

  def close(self):
    """ Close progress bar window. """
    GUI.invoke_after(0.1, super(PulsableProgressDialog, self).close)
    sleep(0.2)

class UpdateView(HasTraits):

  piksi_stm_vers = String('Waiting for Piksi to send settings...')
  newest_stm_vers = String('Downloading Newest Firmware info...')
  piksi_nap_vers = String('Waiting for Piksi to send settings...')
  newest_nap_vers = String('Downloading Newest Firmware info...')
  local_console_vers = String(CONSOLE_VERSION)
  newest_console_vers = String('Downloading Newest Console info...')

  erase_stm = Bool(True)
  erase_en = Bool(True)

  update_firmware = Button(label='Update Piksi Firmware')
  updating = Bool(False)
  update_en = Bool(False)

  download_firmware = Button(label='Download Newest Firmware Files')
  downloading = Bool(False)
  download_fw_en = Bool(True)

  stm_fw = Instance(IntelHexFileDialog)
  nap_fw = Instance(IntelHexFileDialog)

  stream = Instance(OutputStream)

  view = View(
    VGroup(
      HGroup(
        VGroup(
          Item('piksi_stm_vers', label='Piksi STM Firmware Version'),
          Item('newest_stm_vers', label='Newest STM Firmware Version'),
          Item('piksi_nap_vers', label='Piksi NAP Firmware Version'),
          Item('newest_nap_vers', label='Newest NAP Firmware Version'),
          Item('local_console_vers', label='Local Piksi Console Version'),
          Item('newest_console_vers', label='Newest Piksi Console Version'),
        ),
        VGroup(
          Item('stm_fw', style='custom', label='STM Firmware File', \
               enabled_when='download_fw_en'),
          Item('nap_fw', style='custom', label='NAP Firmware File', \
               enabled_when='download_fw_en'),
          Item('erase_stm', label='Erase Entire STM flash', \
               enabled_when='erase_en'),
        ),
      ),
      UItem('download_firmware', enabled_when='download_fw_en'),
      UItem('update_firmware', enabled_when='update_en'),
      Item(
        'stream',
        style='custom',
        editor=InstanceEditor(),
        label='Update Status',
      ),
    )
  )

  def __init__(self, link, prompt=True):
    """
    Traits tab with UI for updating Piksi firmware.

    Parameters
    ----------
    link : sbp.client.handler.Handler
      Link for SBP transfer to/from Piksi.
    prompt : bool
      Prompt user to update console/firmware if out of date.
    """
    self.link = link
    self.settings = {}
    self.prompt = prompt
    self.python_console_cmds = {
      'update': self

    }
    self.update_dl = None
    self.stm_fw = IntelHexFileDialog('STM')
    self.stm_fw.on_trait_change(self._manage_enables, 'status')
    self.nap_fw = IntelHexFileDialog('M25')
    self.nap_fw.on_trait_change(self._manage_enables, 'status')
    self.stream = OutputStream()
    self.get_latest_version_info()

  def _manage_enables(self):
    """ Manages whether traits widgets are enabled in the UI or not. """
    if self.updating == True or self.downloading == True:
      self.update_en = False
      self.download_fw_en = False
    else:
      self.download_fw_en = True
      if self.stm_fw.ihx != None and self.nap_fw.ihx != None:
        self.update_en = True
      else:
        self.update_en = False

    if self.updating == True:
      self.erase_en = False
    else:
      self.erase_en = True

  def _updating_changed(self):
    """ Handles self.updating trait being changed. """
    self._manage_enables()

  def _downloading_changed(self):
    """ Handles self.downloading trait being changed. """
    self._manage_enables()

  def _write(self, text):
    """
    Stream style write function. Allows flashing debugging messages to be
    routed to embedded text console.

    Parameters
    ----------
    text : string
      Text to be written to screen.
    """
    self.stream.write(text)
    self.stream.write('\n')
    self.stream.flush()

  def _update_firmware_fired(self):
    """
    Handle update_firmware button. Starts thread so as not to block the GUI
    thread.
    """
    try:
      if self._firmware_update_thread.is_alive():
        return
    except AttributeError:
      pass

    self._firmware_update_thread = Thread(target=self.manage_firmware_updates)
    self._firmware_update_thread.start()

  def _download_firmware(self):
    """ Download latest firmware from swiftnav.com. """
    self._write('')

    # Check that we received the index file from the website.
    if self.update_dl == None:
      self._write("Error: Can't download firmware files")
      return

    self.downloading = True

    status = 'Downloading Newest Firmware...'
    self.nap_fw.clear(status)
    self.stm_fw.clear(status)
    self._write(status)

    # Get firmware files from Swift Nav's website, save to disk, and load.
    try:
      self._write('Downloading Newest NAP firmware')
      filepath = self.update_dl.download_nap_firmware()
      self._write('Saved file to %s' % filepath)
      self.nap_fw.load_ihx(filepath)
    except AttributeError:
      self.nap_fw.clear("Error downloading firmware")
      self._write("Error downloading firmware: index file not downloaded yet")
    except KeyError:
      self.nap_fw.clear("Error downloading firmware")
      self._write("Error downloading firmware: URL not present in index")
    except URLError:
      self.nap_fw.clear("Error downloading firmware")
      self._write("Error: Failed to download latest NAP firmware from Swift Navigation's website")

    try:
      self._write('Downloading Newest STM firmware')
      filepath = self.update_dl.download_stm_firmware()
      self._write('Saved file to %s' % filepath)
      self.stm_fw.load_ihx(filepath)
    except AttributeError:
      self.stm_fw.clear("Error downloading firmware")
      self._write("Error downloading firmware: index file not downloaded yet")
    except KeyError:
      self.stm_fw.clear("Error downloading firmware")
      self._write("Error downloading firmware: URL not present in index")
    except URLError:
      self.stm_fw.clear("Error downloading firmware")
      self._write("Error: Failed to download latest STM firmware from Swift Navigation's website")

    self.downloading = False

  def _download_firmware_fired(self):
    """
    Handle download_firmware button. Starts thread so as not to block the GUI
    thread.
    """
    try:
      if self._download_firmware_thread.is_alive():
        return
    except AttributeError:
      pass

    self._download_firmware_thread = Thread(target=self._download_firmware)
    self._download_firmware_thread.start()

  def compare_versions(self):
    """
    To be called after latest Piksi firmware info has been received from
    device, to decide if current firmware on Piksi is out of date. Starts a
    thread so as not to block GUI thread.
    """
    try:
      if self._compare_versions_thread.is_alive():
        return
    except AttributeError:
      pass

    self._compare_versions_thread = Thread(target=self._compare_versions)
    self._compare_versions_thread.start()

  def _compare_versions(self):
    """
    Compares version info between received firmware version / current console
    and firmware / console info from website to decide if current firmware or
    console is out of date. Prompt user to update if so.
    """
    # Check that settings received from Piksi contain FW versions.
    try:
      self.piksi_stm_vers = \
        self.settings['system_info']['firmware_version'].value
      self.piksi_nap_vers = \
        self.settings['system_info']['nap_version'].value
    except KeyError:
      self._write("\nError: Settings received from Piksi don't contain firmware version keys. Please contact Swift Navigation.\n")
      return

    # Check that we received the index file from the website.
    if self.update_dl == None:
      self._write("Error: No website index to use to compare versions with local firmware")
      return

    # Check if console is out of date and notify user if so.
    if self.prompt:
      local_console_version = parse_version(CONSOLE_VERSION)
      remote_console_version = parse_version(self.newest_console_vers)
      self.console_outdated = remote_console_version > local_console_version

      if self.console_outdated:
        console_outdated_prompt = \
            prompt.CallbackPrompt(
                                  title="Piksi Console Outdated",
                                  actions=[prompt.close_button],
                                 )

        console_outdated_prompt.text = \
            "Your Piksi Console is out of date and may be incompatible\n" + \
            "with current firmware. We highly recommend upgrading to\n" + \
            "ensure proper behavior.\n\n" + \
            "Please visit http://downloads.swiftnav.com to\n" + \
            "download the newest version.\n\n" + \
            "Local Console Version :\n\t" + \
                CONSOLE_VERSION + \
            "\nNewest Console Version :\n\t" + \
                self.update_dl.index['piksi_v2.3.1']['console']['version'] + "\n"

        console_outdated_prompt.run()

    # For timing aesthetics between windows popping up.
    sleep(0.5)

    # Check if firmware is out of date and notify user if so.
    if self.prompt:
      local_stm_version = parse_version(
          self.settings['system_info']['firmware_version'].value)
      remote_stm_version = parse_version(self.newest_stm_vers)

      local_nap_version = parse_version(
          self.settings['system_info']['nap_version'].value)
      remote_nap_version = parse_version(self.newest_nap_vers)

      self.fw_outdated = remote_nap_version > local_nap_version or \
                         remote_stm_version > local_stm_version

      if self.fw_outdated:
        fw_update_prompt = \
            prompt.CallbackPrompt(
                                  title='Firmware Update',
                                  actions=[prompt.close_button]
                                 )

        fw_update_prompt.text = \
            "New Piksi firmware available.\n\n" + \
            "Please use the Firmware Update tab to update.\n\n" + \
            "Newest STM Version :\n\t%s\n\n" % \
                self.update_dl.index['piksi_v2.3.1']['stm_fw']['version'] + \
            "Newest SwiftNAP Version :\n\t%s\n\n" % \
                self.update_dl.index['piksi_v2.3.1']['nap_fw']['version']

        fw_update_prompt.run()

  def get_latest_version_info(self):
    """
    Get latest firmware / console version from website. Starts thread so as not
    to block the GUI thread.
    """
    try:
      if self._get_latest_version_info_thread.is_alive():
        return
    except AttributeError:
      pass

    self._get_latest_version_info_thread = Thread(target=self._get_latest_version_info)
    self._get_latest_version_info_thread.start()

  def _get_latest_version_info(self):
    """ Get latest firmware / console version from website. """
    try:
      self.update_dl = UpdateDownloader()
    except URLError:
      self._write("\nError: Failed to download latest file index from Swift Navigation's website. Please visit our website to check that you're running the latest Piksi firmware and Piksi console.\n")
      return

    # Make sure index contains all keys we are interested in.
    try:
      self.newest_stm_vers = self.update_dl.index['piksi_v2.3.1']['stm_fw']['version']
      self.newest_nap_vers = self.update_dl.index['piksi_v2.3.1']['nap_fw']['version']
      self.newest_console_vers = self.update_dl.index['piksi_v2.3.1']['console']['version']
    except KeyError:
      self._write("\nError: Index downloaded from Swift Navigation's website (%s) doesn't contain all keys. Please contact Swift Navigation.\n" % INDEX_URL)
      return

  # Executed in GUI thread, called from Handler.
  def manage_firmware_updates(self):
    """
    Update Piksi firmware. Erase entire STM flash (other than bootloader)
    if so directed. Flash NAP only if new firmware is available.
    """
    self.updating = True

    self._write('')

    # Erase all of STM's flash (other than bootloader) if box is checked.
    if self.erase_stm:
      text = "Erasing STM"
      self._write(text)
      self.create_flash("STM")
      sectors_to_erase = set(range(self.pk_flash.n_sectors)).difference(set(self.pk_flash.restricted_sectors))
      progress_dialog = PulsableProgressDialog(len(sectors_to_erase), False)
      progress_dialog.title = text
      GUI.invoke_later(progress_dialog.open)
      erase_count = 0
      for s in sorted(sectors_to_erase):
        progress_dialog.progress(erase_count)
        self._write('Erasing %s sector %d' % (self.pk_flash.flash_type,s))
        self.pk_flash.erase_sector(s)
        erase_count += 1
      self.stop_flash()
      self._write("")
      progress_dialog.close()

    # Flash STM.
    text = "Updating STM"
    self._write(text)
    self.create_flash("STM")
    stm_n_ops = self.pk_flash.ihx_n_ops(self.stm_fw.ihx, \
                                        erase = not self.erase_stm)
    progress_dialog = PulsableProgressDialog(stm_n_ops, True)
    progress_dialog.title = text
    GUI.invoke_later(progress_dialog.open)
    # Don't erase sectors if we've already done so above.
    self.pk_flash.write_ihx(self.stm_fw.ihx, self.stream, mod_print=0x40, \
                            elapsed_ops_cb = progress_dialog.progress, \
                            erase = not self.erase_stm)
    self.stop_flash()
    self._write("")
    progress_dialog.close()

    # Flash NAP if out of date.
    try:
      local_nap_version = parse_version(
          self.settings['system_info']['nap_version'].value)
      remote_nap_version = parse_version(self.newest_nap_vers)
      nap_out_of_date = local_nap_version != remote_nap_version
    except KeyError:
      nap_out_of_date = True
    if nap_out_of_date:
      text = "Updating NAP"
      self._write(text)
      self.create_flash("M25")
      nap_n_ops = self.pk_flash.ihx_n_ops(self.nap_fw.ihx)
      progress_dialog = PulsableProgressDialog(nap_n_ops, True)
      progress_dialog.title = text
      GUI.invoke_later(progress_dialog.open)
      self.pk_flash.write_ihx(self.nap_fw.ihx, self.stream, mod_print=0x40, \
                              elapsed_ops_cb = progress_dialog.progress)
      self.stop_flash()
      self._write("")
      progress_dialog.close()

    # Must tell Piksi to jump to application after updating firmware.
    self.link.send(SBP_MSG_BOOTLOADER_JUMP_TO_APP, '\x00')
    self._write("Firmware updates finished.")
    self._write("")

    self.updating = False

  def create_flash(self, flash_type):
    """
    Create flash.Flash instance and set Piksi into bootloader mode, prompting
    user to reset if necessary.

    Parameter
    ---------
    flash_type : string
      Either "STM" or "M25".
    """
    # Reset device if the application is running to put into bootloader mode.
    self.link.send(SBP_MSG_RESET, '')

    self.pk_boot = bootload.Bootloader(self.link)

    self._write("Waiting for bootloader handshake message from Piksi ...")
    reset_prompt = None
    handshake_received = self.pk_boot.wait_for_handshake(1)

    # Prompt user to reset Piksi if we don't receive the handshake message
    # within a reasonable amount of tiime (firmware might be corrupted).
    while not handshake_received:
      reset_prompt = \
        prompt.CallbackPrompt(
                              title="Please Reset Piksi",
                              actions=[prompt.close_button],
                             )

      reset_prompt.text = \
            "You must press the reset button on your Piksi in order\n" + \
            "to update your firmware.\n\n" + \
            "Please press it now.\n\n"

      reset_prompt.run(block=False)

      while not reset_prompt.closed and not handshake_received:
        handshake_received = self.pk_boot.wait_for_handshake(1)

      reset_prompt.kill()
      reset_prompt.wait()

    self.pk_boot.reply_handshake()
    self._write("received bootloader handshake message.")
    self._write("Piksi Onboard Bootloader Version: " + self.pk_boot.version)

    self.pk_flash = flash.Flash(self.link, flash_type)

  def stop_flash(self):
    """
    Stop Flash and Bootloader instances (removes callback from SerialLink).
    """
    self.pk_flash.stop()
    self.pk_boot.stop()



#!/usr/bin/env python

from piksi_tools import __version__
from setuptools import setup
import os

CLASSIFIERS = [
  'Intended Audience :: Developers',
  'Intended Audience :: Science/Research',
  'Operating System :: POSIX :: Linux',
  'Operating System :: MacOS :: MacOS X',
  'Operating System :: Microsoft :: Windows',
  'Programming Language :: Python',
  'Topic :: Scientific/Engineering :: Interface Engine/Protocol Translator',
  'Topic :: Software Development :: Libraries :: Python Modules',
  'Programming Language :: Python :: 2.7',
]

PACKAGES = [
  'piksi_tools',
]

PLATFORMS = [
  'linux',
  'osx',
  'win32',
]

cwd = os.path.abspath(os.path.dirname(__file__))
with open(cwd + '/README.rst') as f:
  readme = f.read()

with open(cwd + '/requirements.txt') as f:
  INSTALL_REQUIRES = [i.strip() for i in f.readlines()]

setup(name='piksi_tools',
      description='Python tools for the Piksi GNSS receiver.',
      long_description=readme,
      version=__version__,
      author='Swift Navigation',
      author_email='dev@swiftnav.com',
      url='https://github.com/swift-nav/piksi_tools',
      classifiers=CLASSIFIERS,
      packages=PACKAGES,
      platforms=PLATFORMS,
      install_requires=INSTALL_REQUIRES,
      use_2to3=False,
      zip_safe=False)



#!/usr/bin/env python

import yaml
import jinja2
import re
from settings_list import SettingsList
import os

swift_nav_style_path = "../libsbp/docs"
environment_variables_to_append=["TEXINPUTS","PATH"]
myenviron = os.environ
for each in environment_variables_to_append:
  try:
    texinputs = myenviron[each]
    print texinputs
    myenviron[each]=".:" + swift_nav_style_path + ":" + texinputs
  except KeyError:
    myenviron[each]=".:" + swift_nav_style_path
settings = SettingsList("settings.yaml")
groups = settings.return_groups()

#Note, these reg exps will not replace the '^' character to allow exponents in the units text field
LATEX_SUBS_ALLOW_EXPONENTS = (
    (re.compile(r'\\'), r'\\textbackslash'),
    (re.compile(r'([{}_#%&$])'), r'\\\1'),
    (re.compile(r'~'), r'\~{}'),
    (re.compile(r'_'), r'_'),
    (re.compile(r'"'), r"''"),
    (re.compile(r'\.\.\.+'), r'\\ldots'),
    (re.compile(r'\n'), r'\\newline ')
)

NO_UNDERSCORE = re.compile(r'_')

# We sometimes need to remove underscores.
# This will remove the latex safe underscore character and replace with a space
def no_us(value):
    newval = value
    try:
      return NO_UNDERSCORE.sub(' ', newval)
    except TypeError:
      pass
    return None

def escape_tex_exp(value):
    newval = value
    try:
      for pattern, replacement in LATEX_SUBS_ALLOW_EXPONENTS:
          newval = pattern.sub(replacement, newval)
      return newval
    except TypeError:
      pass
    return None

jenv = jinja2.Environment(
    block_start_string = '((*',
    block_end_string = '*))',
    variable_start_string = '(((',
    variable_end_string = ')))',
    comment_start_string = '((=',
    comment_end_string = '=))',
    loader=jinja2.FileSystemLoader("./")
)
jenv.filters['escape_tex_exp'] = escape_tex_exp
jenv.filters['no_us'] = no_us


latex_template = jenv.get_template('settings_template.tex')
with open("settings_out.tex", 'w') as f:
  f.write(latex_template.render(groups=sorted(groups), setting=sorted(settings.list_of_dicts), version='v0.15'))

import subprocess

subprocess.Popen(["pdflatex" , "--shell-escape", "settings_out.tex"], env=myenviron).wait()
subprocess.call(["mv" , "settings_out.pdf", "../docs/settings.pdf"])




#!/usr/bin/env python
# Copyright (C) 2011-2014 Swift Navigation Inc.
# Contact: Colin Beighley <colin@swift-nav.com>
#
# This source is subject to the license found in the file 'LICENSE' which must
# be be distributed together with this source. All other rights reserved.
#
# THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
# EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.

import time
import struct
import sys
import serial_link

from sbp.piksi          import SBP_MSG_STM_UNIQUE_ID
from sbp.client.handler import *

class STMUniqueID:

  def __init__(self,link):
    self.unique_id_returned = False
    self.unique_id = None
    self.link = link
    link.add_callback(self.receive_stm_unique_id_callback, SBP_MSG_STM_UNIQUE_ID)

  def receive_stm_unique_id_callback(self,sbp_msg):
    self.unique_id_returned = True
    self.unique_id = struct.unpack('<12B',sbp_msg.payload)

  def get_id(self):
    self.unique_id_returned = False
    self.unique_id = None
    self.link.send(SBP_MSG_STM_UNIQUE_ID, struct.pack("<I",0))
    while not self.unique_id_returned:
      time.sleep(0.1)
    return self.unique_id

def get_args():
  """
  Get and parse arguments.
  """
  import argparse
  parser = argparse.ArgumentParser(description='Acquisition Monitor')
  parser.add_argument("-f", "--ftdi",
                      help="use pylibftdi instead of pyserial.",
                      action="store_true")
  parser.add_argument('-p', '--port',
                      default=[serial_link.SERIAL_PORT], nargs=1,
                      help='specify the serial port to use.')
  parser.add_argument("-b", "--baud",
                      default=[serial_link.SERIAL_BAUD], nargs=1,
                      help="specify the baud rate to use.")
  return parser.parse_args()

def main():
  """
  Get configuration, get driver, and build handler and start it.
  """
  args = get_args()
  port = args.port[0]
  baud = args.baud[0]
  # Driver with context
  with serial_link.get_driver(args.ftdi, port, baud) as driver:
    with Handler(driver.read, driver.write) as link:
      link.start()
      unique_id = STMUniqueID(link).get_id()
      print "STM Unique ID =", "0x" + ''.join(["%02x" % (b) for b in unique_id])

if __name__ == "__main__":
  main()



#!/usr/bin/env python
# Copyright (C) 2011-2014 Swift Navigation Inc.
# Contact: Fergus Noble <fergus@swift-nav.com>
#
# This source is subject to the license found in the file 'LICENSE' which must
# be be distributed together with this source. All other rights reserved.
#
# THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
# EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.

from traits.api import Instance, Dict, HasTraits, Array, Float, on_trait_change, List, Int, Button, Bool
from traitsui.api import Item, View, HGroup, VGroup, ArrayEditor, HSplit, TabularEditor
from traitsui.tabular_adapter import TabularAdapter

import struct
import math
import os
import numpy as np
import datetime

from sbp.piksi    import SBP_MSG_THREAD_STATE, SBP_MSG_UART_STATE
from sbp.standard import SBP_MSG_HEARTBEAT

class SimpleAdapter(TabularAdapter):
    columns = [('Thread Name', 0), ('CPU %',  1), ('Stack Free',  2)]

class ThreadState:
  def from_binary(self, data):
    state = struct.unpack('<20sHI', data)
    self.name = state[0].rstrip('\0')
    if self.name == '':
      self.name = '(no name)'
    self.cpu = 100 * state[1] / 1000.
    self.stack_free = state[2]


class SystemMonitorView(HasTraits):
  python_console_cmds = Dict()

  _threads_table_list = List()
  threads = List()

  uart_a_crc_error_count = Int(0)
  uart_a_io_error_count = Int(0)
  uart_a_rx_buffer = Float(0)
  uart_a_tx_buffer = Float(0)
  uart_a_tx_KBps = Float(0)
  uart_a_rx_KBps = Float(0)

  uart_b_crc_error_count = Int(0)
  uart_b_io_error_count = Int(0)
  uart_b_rx_buffer = Float(0)
  uart_b_tx_buffer = Float(0)
  uart_b_tx_KBps = Float(0)
  uart_b_rx_KBps = Float(0)


  ftdi_crc_error_count = Int(0)
  ftdi_io_error_count = Int(0)
  ftdi_rx_buffer = Float(0)
  ftdi_tx_buffer = Float(0)
  ftdi_tx_KBps = Float(0)
  ftdi_rx_KBps = Float(0)

  msg_obs_avg_latency_ms    = Int(0)
  msg_obs_min_latency_ms    = Int(0)
  msg_obs_max_latency_ms    = Int(0)
  msg_obs_window_latency_ms = Int(0)

  traits_view = View(
    VGroup(
      Item(
        '_threads_table_list', style = 'readonly',
        editor = TabularEditor(adapter=SimpleAdapter()),
        show_label=False, width=0.85,
      ),
      HGroup(
        VGroup(
          Item('msg_obs_window_latency_ms', label='Obs Latency',
            style='readonly', format_str='%dms'),
          Item('msg_obs_avg_latency_ms', label='Obs Latency (Avg ms)',
            style='readonly', format_str='%dms'),
          Item('msg_obs_min_latency_ms', label='Obs Latency (Min ms)',
            style='readonly', format_str='%dms'),
          Item('msg_obs_max_latency_ms', label='Obs Latency (Max ms)',
            style='readonly', format_str='%dms'),
          label='Connection Monitor', show_border=True,
        ),
        VGroup(
          Item('uart_a_crc_error_count', label='CRC Errors', style='readonly'),
          Item('uart_a_io_error_count', label='IO Errors', style='readonly'),
          Item('uart_a_tx_buffer', label='TX Buffer %',
               style='readonly', format_str='%.1f'),
          Item('uart_a_rx_buffer', label='RX Buffer %',
               style='readonly', format_str='%.1f'),
          Item('uart_a_tx_KBps', label='TX KBytes/s',
               style='readonly', format_str='%.2f'),
          Item('uart_a_rx_KBps', label='RX KBytes/s',
               style='readonly', format_str='%.2f'),
          label='UART A', show_border=True,
        ),
        VGroup(
          Item('uart_b_crc_error_count', label='CRC Errors', style='readonly'),
          Item('uart_b_io_error_count', label='IO Errors', style='readonly'),
          Item('uart_b_tx_buffer', label='TX Buffer %',
               style='readonly', format_str='%.1f'),
          Item('uart_b_rx_buffer', label='RX Buffer %',
               style='readonly', format_str='%.1f'),
          Item('uart_b_tx_KBps', label='TX KBytes/s',
               style='readonly', format_str='%.2f'),
          Item('uart_b_rx_KBps', label='RX KBytes/s',
               style='readonly', format_str='%.2f'),
          label='UART B', show_border=True,
        ),
        VGroup(
          Item('ftdi_crc_error_count', label='CRC Errors', style='readonly'),
          Item('ftdi_io_error_count', label='IO Errors', style='readonly'),
          Item('ftdi_tx_buffer', label='TX Buffer %',
               style='readonly', format_str='%.1f'),
          Item('ftdi_rx_buffer', label='RX Buffer %',
               style='readonly', format_str='%.1f'),
          Item('ftdi_tx_KBps', label='TX KBytes/s',
               style='readonly', format_str='%.2f'),
          Item('ftdi_rx_KBps', label='RX KBytes/s',
               style='readonly', format_str='%.2f'),
          label='USB UART', show_border=True,
        ),
      ),
    )
  )

  def update_threads(self):
    self._threads_table_list = [(thread_name, state.cpu, state.stack_free)
      for thread_name, state in sorted(
        self.threads, key=lambda x: x[1].cpu, reverse=True)]

  def heartbeat_callback(self, sbp_msg):
    self.update_threads()
    self.threads = []

  def thread_state_callback(self, sbp_msg):
    th = ThreadState()
    th.from_binary(sbp_msg.payload)
    self.threads.append((th.name, th))

  def uart_state_callback(self, sbp_msg):
    state = struct.unpack('<ffHHBBffHHBBffHHBBiiii', sbp_msg.payload)
    uarta = state[0:6]
    uartb = state[6:12]
    ftdi = state[12:18]

    self.uart_a_tx_KBps, self.uart_a_rx_KBps = uarta[0:2]
    self.uart_a_crc_error_count = uarta[2]
    self.uart_a_io_error_count = uarta[3]
    self.uart_a_tx_buffer, self.uart_a_rx_buffer = map(
      lambda x: 100.0 * x / 255.0, uarta[4:6])

    self.uart_b_tx_KBps, self.uart_b_rx_KBps = uartb[0:2]
    self.uart_b_crc_error_count = uartb[2]
    self.uart_b_io_error_count = uartb[3]
    self.uart_b_tx_buffer, self.uart_b_rx_buffer = map(
      lambda x: 100.0 * x / 255.0, uartb[4:6])

    self.ftdi_tx_KBps, self.ftdi_rx_KBps = ftdi[0:2]
    self.ftdi_crc_error_count = ftdi[2]
    self.ftdi_io_error_count = ftdi[3]
    self.ftdi_tx_buffer, self.ftdi_rx_buffer = map(
      lambda x: 100.0 * x / 255.0, ftdi[4:6])

    self.msg_obs_avg_latency_ms = state[-4]
    self.msg_obs_min_latency_ms = state[-3]
    self.msg_obs_max_latency_ms = state[-2]
    self.msg_obs_window_latency_ms = state[-1]


  def __init__(self, link):
    super(SystemMonitorView, self).__init__()

    self.link = link
    self.link.add_callback(self.heartbeat_callback, SBP_MSG_HEARTBEAT)
    self.link.add_callback(self.thread_state_callback, SBP_MSG_THREAD_STATE)
    self.link.add_callback(self.uart_state_callback, SBP_MSG_UART_STATE)


    self.python_console_cmds = {
      'mon': self
    }
